@@ -351,14 +351,23 @@ gchar *conv_codeset_strdup(const gchar *inbuf,
 		}
 	}
 
+	/* don't convert if src and dest codeset are identical */
+	if (src_codeset && dest_codeset &&
+	    !strcasecmp(src_codeset, dest_codeset))
+		return g_strdup(inbuf);
+
 #if HAVE_LIBJCONV
 	if (src_codeset) {
 		codesets = &src_codeset;
 		n_codesets = 1;
 	} else
 		codesets = jconv_info_get_pref_codesets(&n_codesets);
-	if (!dest_codeset)
+	if (!dest_codeset) {
 		dest_codeset = conv_get_current_charset_str();
+		/* don't convert if current codeset is US-ASCII */
+		if (!strcasecmp(dest_codeset, CS_US_ASCII))
+			return g_strdup(inbuf);
+	}
 
 	if (jconv_alloc_conv(inbuf, strlen(inbuf), &buf, &len,
 			     codesets, n_codesets,
@@ -370,7 +379,7 @@ gchar *conv_codeset_strdup(const gchar *inbuf,
 #else /* !HAVE_LIBJCONV */
 	if (src_codeset) {
 		if (!strcasecmp(src_codeset, CS_EUC_JP) ||
-		    !strcasecmp(src_codeset, "EUCJP"))
+		    !strcasecmp(src_codeset, CS_EUCJP))
 			src_charset = C_EUC_JP;
 		else if (!strcasecmp(src_codeset, CS_SHIFT_JIS) ||
 			 !strcasecmp(src_codeset, "SHIFT-JIS") ||
@@ -426,7 +435,7 @@ CodeConvFunc conv_get_code_conv_func(const gchar *charset)
 		 !strcasecmp(charset, "X-SJIS"))
 		code_conv = conv_sjistodisp;
 	else if (!strcasecmp(charset, CS_EUC_JP) ||
-		 !strcasecmp(charset, "EUCJP"))
+		 !strcasecmp(charset, CS_EUCJP))
 		code_conv = conv_euctodisp;
 	else
 		code_conv = conv_noconv;
@@ -439,6 +448,7 @@ static const struct {
 	gchar *const name;
 } charsets[] = {
 	{C_US_ASCII,		CS_US_ASCII},
+	{C_US_ASCII,		CS_ANSI_X3_4_1968},
 	{C_UTF_8,		CS_UTF_8},
 	{C_ISO_8859_1,		CS_ISO_8859_1},
 	{C_ISO_8859_2,		CS_ISO_8859_2},
@@ -457,6 +467,7 @@ static const struct {
 	{C_ISO_2022_JP,		CS_ISO_2022_JP},
 	{C_ISO_2022_JP_2,	CS_ISO_2022_JP_2},
 	{C_EUC_JP,		CS_EUC_JP},
+	{C_EUC_JP,		CS_EUCJP},
 	{C_SHIFT_JIS,		CS_SHIFT_JIS},
 	{C_ISO_2022_KR,		CS_ISO_2022_KR},
 	{C_EUC_KR,		CS_EUC_KR},
@@ -521,8 +532,9 @@ static const struct {
 	{"lt_LT"		, C_ISO_8859_13},
 	{"lv_LV"		, C_ISO_8859_13},
 
-	{"C"		, C_US_ASCII},
-	{"POSIX"	, C_US_ASCII},
+	{"C"			, C_US_ASCII},
+	{"POSIX"		, C_US_ASCII},
+	{"ANSI_X3.4-1968"	, C_US_ASCII},
 };
 #endif /* !HAVE_LIBJCONV */
 
@@ -619,16 +631,12 @@ CharSet conv_get_current_charset(void)
 
 const gchar *conv_get_current_charset_str(void)
 {
-#if HAVE_LIBJCONV
-	return jconv_info_get_current_codeset();
-#else
 	static const gchar *codeset = NULL;
 
 	if (!codeset)
 		codeset = conv_get_charset_str(conv_get_current_charset());
 
 	return codeset ? codeset : "US-ASCII";
-#endif
 }
 
 CharSet conv_get_outgoing_charset(void)
@@ -649,7 +657,7 @@ CharSet conv_get_outgoing_charset(void)
 	/* skip US-ASCII and UTF-8 */
 	pref_codesets = jconv_info_get_pref_codesets(&n_pref_codesets);
 	for (i = 0; i < n_pref_codesets; i++) {
-		for (j = 2; j < sizeof(charsets) / sizeof(charsets[0]); j++) {
+		for (j = 3; j < sizeof(charsets) / sizeof(charsets[0]); j++) {
 			if (!strcasecmp(pref_codesets[i], charsets[j].name)) {
 				out_charset = charsets[j].charset;
 				return out_charset;
@@ -794,7 +802,7 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 	gchar *mimehdr_enctype = "?B?";
 	const gchar *mimehdr_charset;
 
-	//g_print("src = %s\n", src);
+	/* g_print("src = %s\n", src); */
 	mimehdr_charset = conv_get_outgoing_charset_str();
 
 	/* convert to wide-character string */
@@ -804,7 +812,6 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 		strlen(mimehdr_charset) + strlen(mimehdr_enctype);
 	mimehdr_begin_len = strlen(mimehdr_init) +
 		strlen(mimehdr_charset) + strlen(mimehdr_enctype);
-	//line_len = 1;
 	line_len = header_len;
 	destp = dest;
 	*dest = '\0';
@@ -867,8 +874,8 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 				}
 				/* g_free(raw); */
 				src_codeset = conv_get_current_charset_str();
-				//printf ("tmp = %s, tlen = %d, mbl\n",
-				//      tmp, tlen, mbl);
+				/* printf ("tmp = %s, tlen = %d, mbl\n",
+					tmp, tlen, mbl); */
 				if (jconv_alloc_conv(tmp, tlen + mbl,
 						     &raw_new, &raw_new_len,
 						     &src_codeset, 1,
@@ -876,6 +883,7 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 				    != 0) {
 					g_warning("can't convert\n");
 					tmpp[0] = '\0';
+					wtmpp++;
 					continue;
 				}
 				if (!str_ascii) {
@@ -925,8 +933,8 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 
 				wtmpp++;
 			}
-			//g_print("tmp = %s, tlen = %d, mb_seqlen = %d\n",
-			//      tmp, tlen, mb_seqlen);
+			/* g_print("tmp = %s, tlen = %d, mb_seqlen = %d\n",
+				tmp, tlen, mb_seqlen); */
 
 			if (tlen == 0 || raw_len == 0) {
 				g_free(tmp);
@@ -962,7 +970,7 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 
 			g_free(tmp);
 			g_free(raw);
-			//g_print("line_len = %d\n\n", line_len);
+			/* g_print("line_len = %d\n\n", line_len); */
 		} while (*wtmpp != (wchar_t)0);
 
 		while (iswspace(*wsrcp)) {
@@ -981,7 +989,7 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 
 	g_free(wsrc);
 
-	//g_print("dest = %s\n", dest);
+	/* g_print("dest = %s\n", dest); */
 }
 #else /* !HAVE_LIBJCONV */
 
@@ -999,7 +1007,7 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 	gchar *mimehdr_enctype = "?B?";
 	const gchar *mimehdr_charset;
 
-	//g_print("src = %s\n", src);
+	/* g_print("src = %s\n", src); */
 	mimehdr_charset = conv_get_outgoing_charset_str();
 	if (strcmp(mimehdr_charset, "ISO-2022-JP") != 0) {
 		/* currently only supports Japanese */
@@ -1014,7 +1022,6 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 		      strlen(mimehdr_charset) + strlen(mimehdr_enctype);
 	mimehdr_begin_len = strlen(mimehdr_init) +
 			    strlen(mimehdr_charset) + strlen(mimehdr_enctype);
-	//line_len = 1;
 	line_len = header_len;
 	destp = dest;
 	*dest = '\0';
@@ -1113,8 +1120,8 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 
 				wtmpp++;
 			}
-			//g_print("tmp = %s, tlen = %d, mb_seqlen = %d\n",
-			//	tmp, tlen, mb_seqlen);
+			/* g_print("tmp = %s, tlen = %d, mb_seqlen = %d\n",
+				tmp, tlen, mb_seqlen); */
 
 			if (tlen == 0) {
 				g_free(tmp);
@@ -1155,7 +1162,7 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 			}
 
 			g_free(tmp);
-			//g_print("line_len = %d\n\n", line_len);
+			/* g_print("line_len = %d\n\n", line_len); */
 		} while (*wtmpp != (wchar_t)0);
 
 		while (iswspace(*wsrcp)) {
@@ -1174,6 +1181,6 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 
 	g_free(wsrc);
 
-	//g_print("dest = %s\n", dest);
+	/* g_print("dest = %s\n", dest); */
 }
 #endif /* HAVE_LIBJCONV */
