@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -77,6 +77,7 @@ struct _CodeConverter
 
 #define CS_AUTO			"AUTO"
 #define CS_US_ASCII		"US-ASCII"
+#define CS_ANSI_X3_4_1968	"ANSI_X3.4-1968"
 #define CS_UTF_8		"UTF-8"
 #define CS_ISO_8859_1		"ISO-8859-1"
 #define CS_ISO_8859_2		"ISO-8859-2"
@@ -95,6 +96,7 @@ struct _CodeConverter
 #define CS_ISO_2022_JP		"ISO-2022-JP"
 #define CS_ISO_2022_JP_2	"ISO-2022-JP-2"
 #define CS_EUC_JP		"EUC-JP"
+#define CS_EUCJP		"EUCJP"
 #define CS_SHIFT_JIS		"Shift_JIS"
 #define CS_ISO_2022_KR		"ISO-2022-KR"
 #define CS_EUC_KR		"EUC-KR"
