@@ -110,9 +110,12 @@ struct _Compose
 
 	GtkItemFactory *popupfactory;
 
+	GtkWidget *tmpl_menu;
+
 	ComposeMode mode;
 
 	MsgInfo *targetinfo;
+	MsgInfo *replyinfo;
 
 	gchar	*replyto;
 	gchar	*cc;
@@ -179,4 +182,6 @@ void compose_entry_append	(Compose	  *compose,
 				 const gchar	  *address,
 				 ComposeEntryType  type);
 
+void compose_reflect_prefs_all	(void);
+
 #endif /* __COMPOSE_H__ */
