@@ -47,6 +47,8 @@
 #define CUSTOM_HEADER_RC	"customheaderrc"
 #define DISPLAY_HEADER_RC	"dispheaderrc"
 #define MENU_RC			"menurc"
+#define COMMAND_HISTORY		"command_history"
+#define TEMPLATE_DIR		"templates"
 #define NEWSGROUP_LIST		".newsgroup_list"
 #define ADDRESS_BOOK		"addressbook.xml"
 #define MANUAL_HTML_INDEX	"sylpheed.html"
@@ -87,6 +89,7 @@
 #define FOLDER_SPACING			4
 #define COLOR_DIM			35000
 #define UI_REFRESH_INTERVAL		50000
+#define MAX_HISTORY_SIZE		16
 
 #define DEFAULT_NORMAL_FONT	"-*-helvetica-medium-r-normal--12-*," \
 				"-*-*-medium-r-normal--12-*-*-*-*-*-*-*,*"
