@@ -142,6 +142,7 @@ FolderItem *folder_item_new(const gchar *name, const gchar *path)
 	item->no_sub = FALSE;
 	item->no_select = FALSE;
 	item->collapsed = FALSE;
+	item->threaded  = TRUE;
 	item->parent = NULL;
 	item->folder = NULL;
 	item->data = NULL;
@@ -618,6 +619,23 @@ gint folder_item_remove_msg(FolderItem *item, gint num)
 	return ret;
 }
 
+gint folder_item_remove_msgs(FolderItem *item, GSList *msglist)
+{
+	gint ret = 0;
+
+	g_return_val_if_fail(item != NULL, -1);
+
+	while (msglist != NULL) {
+		MsgInfo *msginfo = (MsgInfo *)msglist->data;
+
+		ret = folder_item_remove_msg(item, msginfo->msgnum);
+		if (ret != 0) break;
+		msglist = msglist->next;
+	}
+
+	return ret;
+}
+
 gint folder_item_remove_all_msg(FolderItem *item)
 {
 	Folder *folder;
@@ -795,7 +813,8 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 	const gchar *name = NULL;
 	const gchar *path = NULL;
 	PrefsAccount *account = NULL;
-	gboolean no_sub = FALSE, no_select = FALSE, collapsed = FALSE;
+	gboolean no_sub = FALSE, no_select = FALSE, collapsed = FALSE,
+		 threaded = TRUE;
 	gint mtime = 0, new = 0, unread = 0, total = 0;
 
 	g_return_val_if_fail(node->data != NULL, FALSE);
@@ -847,6 +866,8 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 			no_select = *attr->value == '1' ? TRUE : FALSE;
 		else if (!strcmp(attr->name, "collapsed"))
 			collapsed = *attr->value == '1' ? TRUE : FALSE;
+		else if (!strcmp(attr->name, "threaded"))
+			threaded =  *attr->value == '1' ? TRUE : FALSE;
 	}
 
 	item = folder_item_new(name, path);
@@ -859,6 +880,7 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 	item->no_sub = no_sub;
 	item->no_select = no_select;
 	item->collapsed = collapsed;
+	item->threaded  = threaded;
 	item->parent = FOLDER_ITEM(node->parent->data);
 	item->folder = folder;
 	switch (stype) {
@@ -884,7 +906,7 @@ static gboolean folder_read_folder_func(GNode *node, gpointer data)
 	const gchar *name = NULL;
 	const gchar *path = NULL;
 	PrefsAccount *account = NULL;
-	gboolean collapsed = FALSE;
+	gboolean collapsed = FALSE, threaded = TRUE;
 
 	if (g_node_depth(node) != 2) return FALSE;
 	g_return_val_if_fail(node->data != NULL, FALSE);
@@ -921,6 +943,8 @@ static gboolean folder_read_folder_func(GNode *node, gpointer data)
 						attr->value);
 		} else if (!strcmp(attr->name, "collapsed"))
 			collapsed = *attr->value == '1' ? TRUE : FALSE;
+		else if (!strcmp(attr->name, "threaded"))
+			threaded = *attr->value == '1' ? TRUE : FALSE;
 	}
 
 	folder = folder_new(type, name, path);
@@ -933,6 +957,7 @@ static gboolean folder_read_folder_func(GNode *node, gpointer data)
 	folder->node = node;
 	folder_add(folder);
 	FOLDER_ITEM(node->data)->collapsed = collapsed;
+	FOLDER_ITEM(node->data)->threaded  = threaded;
 
 	g_node_traverse(node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
 			folder_build_tree, folder);
@@ -1008,8 +1033,12 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 			fputs(" no_select=\"1\"", fp);
 		if (item->collapsed && node->children)
 			fputs(" collapsed=\"1\"", fp);
+		if (item->threaded)
+			fputs(" threaded=\"1\"", fp);
+		else
+			fputs(" threaded=\"0\"", fp);
 		fprintf(fp,
-			" mtime=\"%ld\" new=\"%d\" unread=\"%d\" total=\"%d\"",
+			" mtime=\"%lu\" new=\"%d\" unread=\"%d\" total=\"%d\"",
 			item->mtime, item->new, item->unread, item->total);
 	}
 
