@@ -208,9 +208,10 @@ struct _FolderItem
 	gint last_num;
 
 	/* special flags */
-	guint no_sub    : 1; /* no child allowed? */
-	guint no_select : 1; /* not selectable?   */
-	guint collapsed : 1; /* collapsed item    */
+	guint no_sub    : 1; /* no child allowed?    */
+	guint no_select : 1; /* not selectable?      */
+	guint collapsed : 1; /* collapsed item       */
+	guint threaded  : 1; /* threaded folder view */
 
 	FolderItem *parent;
 
@@ -279,6 +280,8 @@ gint   folder_item_copy_msgs_with_dest	(FolderItem	*dest,
 					 GSList		*msglist);
 gint   folder_item_remove_msg		(FolderItem	*item,
 					 gint		 num);
+gint   folder_item_remove_msgs		(FolderItem	*item,
+					 GSList		*msglist);
 gint   folder_item_remove_all_msg	(FolderItem	*item);
 gboolean folder_item_is_msg_changed	(FolderItem	*item,
 					 MsgInfo	*msginfo);
