@@ -54,6 +54,7 @@
 #include "prefs_account.h"
 #include "account.h"
 #include "folder.h"
+#include "inc.h"
 
 #include "pixmaps/inbox.xpm"
 #include "pixmaps/outbox.xpm"
@@ -213,6 +214,8 @@ static GtkItemFactoryEntry folderview_mail_popup_entries[] =
 	{N_("/_Delete folder"),		NULL, folderview_delete_folder_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Update folder tree"),	NULL, folderview_update_tree_cb, 0, NULL},
+	{N_("/Re_scan folder tree"),	NULL, folderview_update_tree_cb, 1, NULL},
+	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/Remove _mailbox"),	NULL, folderview_remove_mailbox_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Property..."),		NULL, NULL, 0, NULL}
@@ -225,6 +228,8 @@ static GtkItemFactoryEntry folderview_imap_popup_entries[] =
 	{N_("/_Delete folder"),		NULL, folderview_rm_imap_folder_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Update folder tree"),	NULL, folderview_update_tree_cb, 0, NULL},
+	{N_("/Re_scan folder tree"),	NULL, folderview_update_tree_cb, 1, NULL},
+	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/Remove _IMAP4 account"),	NULL, folderview_rm_imap_server_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Property..."),		NULL, NULL, 0, NULL}
@@ -611,7 +616,8 @@ void folderview_update_tree(Folder *folder)
 
 	if (!folder->scan_tree) return;
 
-	window = label_window_create(_("Updating folder tree..."));
+	inc_lock();
+	window = label_window_create(_("Rescanning folder tree..."));
 
 	folder_set_ui_func(folder, folderview_scan_tree_func, NULL);
 	folder->scan_tree(folder);
@@ -621,6 +627,7 @@ void folderview_update_tree(Folder *folder)
 	folderview_set_all();
 
 	gtk_widget_destroy(window);
+	inc_unlock();
 }
 
 void folderview_update_all(void)
@@ -628,7 +635,8 @@ void folderview_update_all(void)
 	GList *list;
 	GtkWidget *window;
 
-	window = label_window_create(_("Updating all folders..."));
+	inc_lock();
+	window = label_window_create(_("Rescanning all folder trees..."));
 
 	list = folder_get_list();
 	for (; list != NULL; list = list->next) {
@@ -643,6 +651,7 @@ void folderview_update_all(void)
 	folder_write_list();
 	folderview_set_all();
 	gtk_widget_destroy(window);
+	inc_unlock();
 }
 
 void folderview_update_all_node(void)
@@ -652,14 +661,15 @@ void folderview_update_all_node(void)
 	FolderView *folderview;
 	GtkCTree *ctree;
 	GtkCTreeNode *node;
-	GtkWidget *window;
-
-	window = label_window_create(_("Updating all folders..."));
 
 	for (list = folderview_list; list != NULL; list = list->next) {
 		folderview = (FolderView *)list->data;
 		ctree = GTK_CTREE(folderview->ctree);
 
+		inc_lock();
+		main_window_lock(folderview->mainwin);
+		gtk_widget_set_sensitive(folderview->ctree, FALSE);
+
 		for (node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
 		     node != NULL; node = gtkut_ctree_node_next(ctree, node)) {
 			item = gtk_ctree_node_get_row_data(ctree, node);
@@ -670,10 +680,13 @@ void folderview_update_all_node(void)
 			folder_item_scan(item);
 			folderview_update_node(folderview, node);
 		}
+
+		gtk_widget_set_sensitive(folderview->ctree, TRUE);
+		main_window_unlock(folderview->mainwin);
+		inc_unlock();
 	}
 
 	folder_write_list();
-	gtk_widget_destroy(window);
 }
 
 static gboolean folderview_search_new_recursive(GtkCTree *ctree,
@@ -1112,6 +1125,13 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 	gint prev_row = -1, row = -1, column = -1;
 	FolderItem *item;
 	Folder *folder;
+	GtkWidget *popup;
+	gboolean new_folder    = FALSE;
+	gboolean rename_folder = FALSE;
+	gboolean delete_folder = FALSE;
+	gboolean update_tree   = FALSE;
+	gboolean rescan_tree   = FALSE;
+	gboolean remove_tree   = FALSE;
 
 	if (!event) return;
 
@@ -1153,63 +1173,55 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 	g_return_if_fail(item->folder != NULL);
 	folder = item->folder;
 
-	menu_set_insensitive_all(GTK_MENU_SHELL(folderview->mail_popup));
-	menu_set_insensitive_all(GTK_MENU_SHELL(folderview->imap_popup));
-	menu_set_insensitive_all(GTK_MENU_SHELL(folderview->news_popup));
-
-	if (FOLDER_IS_LOCAL(folder) && item->parent == NULL) {
-		menu_set_sensitive(folderview->mail_factory,
-				   "/Create new folder...", TRUE);
-		menu_set_sensitive(folderview->mail_factory,
-				   "/Update folder tree", TRUE);
-		menu_set_sensitive(folderview->mail_factory,
-				   "/Remove mailbox", TRUE);
-	} else if (FOLDER_IS_LOCAL(folder) && item->stype != F_NORMAL) {
-		menu_set_sensitive(folderview->mail_factory,
-				   "/Create new folder...", TRUE);
-	} else if (FOLDER_IS_LOCAL(folder)) {
-		menu_set_sensitive(folderview->mail_factory,
-				   "/Create new folder...", TRUE);
-		menu_set_sensitive(folderview->mail_factory,
-				   "/Rename folder...", TRUE);
-		menu_set_sensitive(folderview->mail_factory,
-				   "/Delete folder", TRUE);
-	} else if (folder->type == F_IMAP && item->parent == NULL) {
-		menu_set_sensitive(folderview->imap_factory,
-				   "/Create new folder...", TRUE);
-		menu_set_sensitive(folderview->imap_factory,
-				   "/Update folder tree", TRUE);
-		menu_set_sensitive(folderview->imap_factory,
-				   "/Remove IMAP4 account", TRUE);
-	} else if (folder->type == F_IMAP && item->stype != F_NORMAL) {
-		menu_set_sensitive(folderview->imap_factory,
-				   "/Create new folder...", TRUE);
-	} else if (folder->type == F_IMAP) {
-		menu_set_sensitive(folderview->imap_factory,
-				   "/Create new folder...", TRUE);
-		menu_set_sensitive(folderview->imap_factory,
-				   "/Delete folder", TRUE);
-	} else if (folder->type == F_NEWS && item->parent == NULL) {
-		menu_set_sensitive(folderview->news_factory,
-				   "/Subscribe to newsgroup...", TRUE);
-		menu_set_sensitive(folderview->news_factory,
-				   "/Remove news account", TRUE);
-	} else if (folder->type == F_NEWS) {
-		menu_set_sensitive(folderview->news_factory,
-				   "/Subscribe to newsgroup...", TRUE);
-		menu_set_sensitive(folderview->news_factory,
-				   "/Remove newsgroup", TRUE);
+	if (folderview->mainwin->lock_count == 0) {
+		new_folder = TRUE;
+		if (item->parent == NULL)
+			update_tree = remove_tree = TRUE;
+		if (FOLDER_IS_LOCAL(folder) || FOLDER_TYPE(folder) == F_IMAP) {
+			if (item->parent == NULL)
+				update_tree = rescan_tree = TRUE;
+			else if (item->stype == F_NORMAL)
+				rename_folder = delete_folder = TRUE;
+		} else if (FOLDER_TYPE(folder) == F_NEWS) {
+			if (item->parent != NULL)
+				delete_folder = TRUE;
+		}
 	}
 
-	if (FOLDER_IS_LOCAL(folder))
-		gtk_menu_popup(GTK_MENU(folderview->mail_popup), NULL, NULL,
-			       NULL, NULL, event->button, event->time);
-	else if (folder->type == F_IMAP)
-		gtk_menu_popup(GTK_MENU(folderview->imap_popup), NULL, NULL,
-			       NULL, NULL, event->button, event->time);
-	else if (folder->type == F_NEWS)
-		gtk_menu_popup(GTK_MENU(folderview->news_popup), NULL, NULL,
-			       NULL, NULL, event->button, event->time);
+#define SET_SENS(factory, name, sens) \
+	menu_set_sensitive(folderview->factory, name, sens)
+
+	if (FOLDER_IS_LOCAL(folder)) {
+		popup = folderview->mail_popup;
+		menu_set_insensitive_all(GTK_MENU_SHELL(popup));
+		SET_SENS(mail_factory, "/Create new folder...", new_folder);
+		SET_SENS(mail_factory, "/Rename folder...", rename_folder);
+		SET_SENS(mail_factory, "/Delete folder", delete_folder);
+		SET_SENS(mail_factory, "/Update folder tree", update_tree);
+		SET_SENS(mail_factory, "/Rescan folder tree", rescan_tree);
+		SET_SENS(mail_factory, "/Remove mailbox", remove_tree);
+	} else if (FOLDER_TYPE(folder) == F_IMAP) {
+		popup = folderview->imap_popup;
+		menu_set_insensitive_all(GTK_MENU_SHELL(popup));
+		SET_SENS(imap_factory, "/Create new folder...", new_folder);
+		SET_SENS(imap_factory, "/Rename folder...", rename_folder);
+		SET_SENS(imap_factory, "/Delete folder", delete_folder);
+		SET_SENS(imap_factory, "/Update folder tree", update_tree);
+		SET_SENS(imap_factory, "/Rescan folder tree", rescan_tree);
+		SET_SENS(imap_factory, "/Remove IMAP4 account", remove_tree);
+	} else if (FOLDER_TYPE(folder) == F_NEWS) {
+		popup = folderview->news_popup;
+		menu_set_insensitive_all(GTK_MENU_SHELL(popup));
+		SET_SENS(news_factory, "/Subscribe to newsgroup...", new_folder);
+		SET_SENS(news_factory, "/Remove newsgroup", delete_folder);
+		SET_SENS(news_factory, "/Remove news account", remove_tree);
+	} else
+		return;
+
+#undef SET_SENS
+
+	gtk_menu_popup(GTK_MENU(popup), NULL, NULL, NULL, NULL,
+		       event->button, event->time);
 }
 
 static void folderview_button_released(GtkWidget *ctree, GdkEventButton *event,
@@ -1400,7 +1412,10 @@ static void folderview_update_tree_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
 
-	folderview_update_tree(item->folder);
+	if (action == 0)
+		folderview_update_all_node();
+	else
+		folderview_update_tree(item->folder);
 }
 
 static void folderview_new_folder_cb(FolderView *folderview, guint action,
