@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +33,11 @@
 #include <gtk/gtkbindings.h>
 #include <stdarg.h>
 
+#if (HAVE_WCTYPE_H && HAVE_WCHAR_H)
+#  include <wchar.h>
+#  include <wctype.h>
+#endif
+
 #include "intl.h"
 #include "gtkutils.h"
 #include "utils.h"
@@ -243,6 +248,28 @@ void gtkut_combo_set_items(GtkCombo *combo, const gchar *str1, ...)
 	g_list_free(combo_items);
 }
 
+gboolean gtkut_text_match_string(GtkText *text, gint pos, wchar_t *wcs,
+				 gint len, gboolean case_sens)
+{
+	gint match_count = 0;
+
+	for (; match_count < len; pos++, match_count++) {
+		if (case_sens) {
+			if (GTK_TEXT_INDEX(text, pos) != wcs[match_count])
+				break;
+		} else {
+			if (towlower(GTK_TEXT_INDEX(text, pos)) !=
+			    towlower(wcs[match_count]))
+				break;
+		}
+	}
+
+	if (match_count == len)
+		return TRUE;
+	else
+		return FALSE;
+}
+
 void gtkut_widget_disable_theme_engine(GtkWidget *widget)
 {
 	GtkStyle *style, *new_style;
@@ -292,8 +319,8 @@ void gtkut_widget_get_uposition(GtkWidget *widget, gint *px, gint *py)
 
 	sx = gdk_screen_width();
 	sy = gdk_screen_height();
-	x %= sx; if (x < 0) x += sx;
-	y %= sy; if (y < 0) y += sy;
+	x %= sx; if (x < 0) x = 0;
+	y %= sy; if (y < 0) y = 0;
 	*px = x;
 	*py = y;
 }
