@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,12 +20,17 @@
 #ifndef __IMAP_H__
 #define __IMAP_H__
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <glib.h>
 
 #include "folder.h"
 #include "session.h"
 
 typedef struct _IMAPSession	IMAPSession;
+typedef struct _IMAPNameSpace	IMAPNameSpace;
 
 #include "prefs_account.h"
 
@@ -38,6 +43,12 @@ struct _IMAPSession
 	gchar *mbox;
 };
 
+struct _IMAPNameSpace
+{
+	gchar *name;
+	gchar separator;
+};
+
 #define IMAP_SUCCESS	0
 #define IMAP_SOCKET	2
 #define IMAP_AUTHFAIL	3
@@ -63,10 +74,18 @@ typedef enum
 #define IMAP_IS_DELETED(flags)	((flags & IMAP_FLAG_DELETED) != 0)
 #define IMAP_IS_DRAFT(flags)	((flags & IMAP_FLAG_DRAFT) != 0)
 
+#if USE_SSL
+Session *imap_session_new		(const gchar	*server,
+					 gushort	 port,
+					 const gchar	*user,
+					 const gchar	*pass,
+					 gboolean	 use_ssl);
+#else
 Session *imap_session_new		(const gchar	*server,
 					 gushort	 port,
 					 const gchar	*user,
 					 const gchar	*pass);
+#endif
 void imap_session_destroy		(IMAPSession	*session);
 void imap_session_destroy_all		(void);
 
@@ -75,23 +94,36 @@ GSList *imap_get_msg_list		(Folder		*folder,
 					 gboolean	 use_cache);
 gchar *imap_fetch_msg			(Folder		*folder,
 					 FolderItem	*item,
-					 gint		 num);
+					 gint		 uid);
+gint imap_add_msg			(Folder		*folder,
+					 FolderItem	*dest,
+					 const gchar	*file,
+					 gboolean	 remove_source);
 
 gint imap_move_msg			(Folder		*folder,
 					 FolderItem	*dest,
 					 MsgInfo	*msginfo);
-
 gint imap_move_msgs_with_dest		(Folder		*folder,
 					 FolderItem	*dest,
 					 GSList		*msglist);
+gint imap_copy_msg			(Folder		*folder,
+					 FolderItem	*dest,
+					 MsgInfo	*msginfo);
+gint imap_copy_msgs_with_dest		(Folder		*folder,
+					 FolderItem	*dest,
+					 GSList		*msglist);
+
 gint imap_remove_msg			(Folder		*folder,
 					 FolderItem	*item,
-					 gint		 num);
+					 gint		 uid);
 gint imap_remove_all_msg		(Folder		*folder,
 					 FolderItem	*item);
 
 void imap_scan_folder			(Folder		*folder,
 					 FolderItem	*item);
+void imap_scan_tree			(Folder		*folder);
+
+gint imap_create_tree			(Folder		*folder);
 
 FolderItem *imap_create_folder		(Folder		*folder,
 					 FolderItem	*parent,
