@@ -396,9 +396,6 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 	gint error_num = 0;
 	gint new_msgs = 0;
 
-	//gtk_label_set_text(GTK_LABEL(inc_dialog->mainwin->statuslabel),
-	//		   _("Receiving"));
-
 	while (inc_dialog->queue_list != NULL) {
 		session = inc_dialog->queue_list->data;
 		pop3_state = session->pop3_state;
@@ -498,8 +495,6 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 		num++;
 	}
 
-	//gtk_label_set_text(GTK_LABEL(inc_dialog->mainwin->statuslabel), "");
-
 	if (error_num) {
 		if (inc_dialog->show_dialog)
 			manage_window_focus_in(inc_dialog->dialog->window,
@@ -625,13 +620,14 @@ static IncState inc_pop3_session_do(IncSession *session)
 				      automaton_input_cb, atm);
 #endif
 
-	gtk_main();
+	while (!atm->terminated)
+		gtk_main_iteration();
 
 	log_verbosity_set(FALSE);
 	recv_set_ui_func(NULL, NULL);
 
 #if USE_THREADS
-	//pthread_join(sockinfo->connect_thr, NULL);
+	/* pthread_join(sockinfo->connect_thr, NULL); */
 #endif
 	automaton_destroy(atm);
 
@@ -640,6 +636,8 @@ static IncState inc_pop3_session_do(IncSession *session)
 
 static gint pop3_automaton_terminate(SockInfo *source, Automaton *atm)
 {
+	if (atm->terminated) return 0;
+
 	if (atm->tag > 0) {
 		gdk_input_remove(atm->tag);
 		atm->tag = 0;
@@ -648,10 +646,8 @@ static gint pop3_automaton_terminate(SockInfo *source, Automaton *atm)
 		gtk_timeout_remove(atm->timeout_tag);
 		atm->timeout_tag = 0;
 	}
-	if (source) {
+	if (source)
 		sock_close(source);
-		gtk_main_quit();
-	}
 
 	atm->terminated = TRUE;
 
@@ -726,7 +722,7 @@ static gint connection_check_cb(Automaton *atm)
 	IncProgressDialog *inc_dialog = state->session->data;
 	SockInfo *sockinfo = state->sockinfo;
 
-	//g_print("connection check\n");
+	/* g_print("connection check\n"); */
 
 	if (sockinfo->state == CONN_LOOKUPFAILED ||
 	    sockinfo->state == CONN_FAILED) {
@@ -734,6 +730,7 @@ static gint connection_check_cb(Automaton *atm)
 		log_warning(_("Can't connect to POP3 server: %s:%d\n"),
 			    sockinfo->hostname, sockinfo->port);
 		pop3_automaton_terminate(sockinfo, atm);
+		state->sockinfo = NULL;
 		return FALSE;
 	} else if (sockinfo->state == CONN_ESTABLISHED) {
 		atm->timeout_tag = 0;
@@ -908,17 +905,20 @@ static void inc_cancel(GtkWidget *widget, gpointer data)
 	IncSession *session = dialog->queue_list->data;
 	SockInfo *sockinfo = session->pop3_state->sockinfo;
 
+	if (!sockinfo || session->atm->terminated == TRUE) return;
+
 #if USE_THREADS
 	if (sockinfo->state == CONN_READY ||
 	    sockinfo->state == CONN_LOOKUPSUCCESS) {
 		pthread_cancel(sockinfo->connect_thr);
-		//pthread_kill(sockinfo->connect_thr, SIGINT);
+		/* pthread_kill(sockinfo->connect_thr, SIGINT); */
 		g_print("connection was cancelled.\n");
 	}
 #endif
 
 	session->pop3_state->inc_state = INC_CANCEL;
 	pop3_automaton_terminate(sockinfo, session->atm);
+	session->pop3_state->sockinfo = NULL;
 }
 
 static gint inc_spool(void)
