@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include <gtk/gtkhbox.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkentry.h>
+#include <gtk/gtkcombo.h>
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkhbbox.h>
 
@@ -43,14 +44,27 @@
 
 #define INPUT_DIALOG_WIDTH	420
 
+typedef enum
+{
+	INPUT_DIALOG_NORMAL,
+	INPUT_DIALOG_INVISIBLE,
+	INPUT_DIALOG_COMBO
+} InputDialogType;
+
 static gboolean ack;
 
+static InputDialogType type;
+
 static GtkWidget *dialog;
 static GtkWidget *msg_label;
 static GtkWidget *entry;
+static GtkWidget *combo;
 static GtkWidget *ok_button;
 
 static void input_dialog_create	(void);
+static gchar *input_dialog_open	(const gchar	*title,
+				 const gchar	*message,
+				 const gchar	*default_string);
 static void input_dialog_set	(const gchar	*title,
 				 const gchar	*message,
 				 const gchar	*default_string);
@@ -66,41 +80,23 @@ static void key_pressed		(GtkWidget	*widget,
 				 GdkEventKey	*event,
 				 gpointer	 data);
 static void entry_activated	(GtkEditable	*editable);
+static void combo_activated	(GtkEditable	*editable);
+
 
 gchar *input_dialog(const gchar *title, const gchar *message,
 		    const gchar *default_string)
 {
-	gchar *str;
-
 	if (dialog && GTK_WIDGET_VISIBLE(dialog)) return NULL;
 
 	if (!dialog)
 		input_dialog_create();
 
-	input_dialog_set(title, message, default_string);
-	gtk_widget_show(dialog);
-	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
-	manage_window_set_transient(GTK_WINDOW(dialog));
-
-	gtk_main();
-
-	manage_window_focus_out(dialog, NULL, NULL);
-	gtk_widget_hide(dialog);
+	type = INPUT_DIALOG_NORMAL;
+	gtk_widget_hide(combo);
+	gtk_widget_show(entry);
 	gtk_entry_set_visibility(GTK_ENTRY(entry), TRUE);
 
-	if (ack) {
-		str = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
-		if (str && *str == '\0') {
-			g_free(str);
-			str = NULL;
-		}
-	} else
-		str = NULL;
-
-	GTK_EVENTS_FLUSH();
-
-	debug_print("return string = %s\n", str ? str : "(none)");
-	return str;
+	return input_dialog_open(title, message, default_string);
 }
 
 gchar *input_dialog_with_invisible(const gchar *title, const gchar *message,
@@ -111,9 +107,37 @@ gchar *input_dialog_with_invisible(const gchar *title, const gchar *message,
 	if (!dialog)
 		input_dialog_create();
 
+	type = INPUT_DIALOG_INVISIBLE;
+	gtk_widget_hide(combo);
+	gtk_widget_show(entry);
 	gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
 
-	return input_dialog(title, message, default_string);
+	return input_dialog_open(title, message, default_string);
+}
+
+gchar *input_dialog_combo(const gchar *title, const gchar *message,
+			  const gchar *default_string, GList *list)
+{
+	if (dialog && GTK_WIDGET_VISIBLE(dialog)) return NULL;
+
+	if (!dialog)
+		input_dialog_create();
+
+	type = INPUT_DIALOG_COMBO;
+	gtk_widget_hide(entry);
+	gtk_widget_show(combo);
+
+	if (!list) {
+		GList empty_list;
+
+		empty_list.data = (gpointer)"";
+		empty_list.next = NULL;
+		empty_list.prev = NULL;
+		gtk_combo_set_popdown_strings(GTK_COMBO(combo), &empty_list);
+	} else
+		gtk_combo_set_popdown_strings(GTK_COMBO(combo), list);
+
+	return input_dialog_open(title, message, default_string);
 }
 
 static void input_dialog_create(void)
@@ -156,6 +180,11 @@ static void input_dialog_create(void)
 	gtk_signal_connect(GTK_OBJECT(entry), "activate",
 			   GTK_SIGNAL_FUNC(entry_activated), NULL);
 
+	combo = gtk_combo_new();
+	gtk_box_pack_start(GTK_BOX(vbox), combo, FALSE, FALSE, 0);
+	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo)->entry), "activate",
+			   GTK_SIGNAL_FUNC(combo_activated), NULL);
+
 	gtkut_button_set_create(&confirm_area,
 				&ok_button,     _("OK"),
 				&cancel_button, _("Cancel"),
@@ -173,20 +202,69 @@ static void input_dialog_create(void)
 	gtk_widget_show_all(GTK_DIALOG(dialog)->vbox);
 }
 
+static gchar *input_dialog_open(const gchar *title, const gchar *message,
+				const gchar *default_string)
+{
+	gchar *str;
+
+	if (dialog && GTK_WIDGET_VISIBLE(dialog)) return NULL;
+
+	if (!dialog)
+		input_dialog_create();
+
+	input_dialog_set(title, message, default_string);
+	gtk_widget_show(dialog);
+	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
+	manage_window_set_transient(GTK_WINDOW(dialog));
+
+	gtk_main();
+
+	manage_window_focus_out(dialog, NULL, NULL);
+	gtk_widget_hide(dialog);
+
+	if (ack) {
+		GtkEditable *editable;
+
+		if (type == INPUT_DIALOG_COMBO)
+			editable = GTK_EDITABLE(GTK_COMBO(combo)->entry);
+		else
+			editable = GTK_EDITABLE(entry);
+
+		str = gtk_editable_get_chars(editable, 0, -1);
+		if (str && *str == '\0') {
+			g_free(str);
+			str = NULL;
+		}
+	} else
+		str = NULL;
+
+	GTK_EVENTS_FLUSH();
+
+	debug_print("return string = %s\n", str ? str : "(none)");
+	return str;
+}
+
 static void input_dialog_set(const gchar *title, const gchar *message,
 			     const gchar *default_string)
 {
+	GtkWidget *entry_;
+
+	if (type == INPUT_DIALOG_COMBO)
+		entry_ = GTK_COMBO(combo)->entry;
+	else
+		entry_ = entry;
+
 	gtk_window_set_title(GTK_WINDOW(dialog), title);
 	gtk_label_set_text(GTK_LABEL(msg_label), message);
 	if (default_string && *default_string)
-		gtk_entry_set_text(GTK_ENTRY(entry), default_string);
+		gtk_entry_set_text(GTK_ENTRY(entry_), default_string);
 	else
-		gtk_entry_set_text(GTK_ENTRY(entry), "");
-	gtk_entry_set_position(GTK_ENTRY(entry), 0);
-	gtk_entry_select_region(GTK_ENTRY(entry), 0, -1);
+		gtk_entry_set_text(GTK_ENTRY(entry_), "");
+	gtk_entry_set_position(GTK_ENTRY(entry_), 0);
+	gtk_entry_select_region(GTK_ENTRY(entry_), 0, -1);
 
 	gtk_widget_grab_focus(ok_button);
-	gtk_widget_grab_focus(entry);
+	gtk_widget_grab_focus(entry_);
 }
 
 static void ok_clicked(GtkWidget *widget, gpointer data)
@@ -222,3 +300,9 @@ static void entry_activated(GtkEditable *editable)
 	ack = TRUE;
 	gtk_main_quit();
 }
+
+static void combo_activated(GtkEditable *editable)
+{
+	ack = TRUE;
+	gtk_main_quit();
+}
