@@ -28,5 +28,9 @@ gchar *input_dialog			(const gchar	*title,
 gchar *input_dialog_with_invisible	(const gchar	*title,
 					 const gchar	*message,
 					 const gchar	*default_string);
+gchar *input_dialog_combo		(const gchar	*title,
+					 const gchar	*message,
+					 const gchar	*default_string,
+					 GList		*list);
 
 #endif /* __INPUTDIALOG_H__ */
