@@ -148,7 +148,7 @@ int main(int argc, char *argv[])
 	gtk_set_locale();
 	gtk_init(&argc, &argv);
 
-#if USE_THREADS
+#if USE_THREADS || USE_LDAP
 	g_thread_init(NULL);
 	if (!g_thread_supported())
 		g_error(_("g_thread is not supported by glib.\n"));
@@ -343,7 +343,7 @@ static gint get_queued_message_num(void)
 	FolderItem *queue;
 
 	queue = folder_get_default_queue();
-	g_return_val_if_fail(queue != NULL, -1);
+	if (!queue) return -1;
 
 	folder_item_scan(queue);
 	return queue->total;
