@@ -48,6 +48,7 @@
 #include "summaryview.h"
 #include "summary_search.h"
 #include "messageview.h"
+#include "message_search.h"
 #include "headerview.h"
 #include "menu.h"
 #include "folder.h"
@@ -59,6 +60,8 @@
 #include "prefs_common.h"
 #include "prefs_filter.h"
 #include "prefs_account.h"
+#include "prefs_summary_column.h"
+#include "prefs_template.h"
 #include "account.h"
 #include "addressbook.h"
 #include "headerwindow.h"
@@ -179,6 +182,10 @@ static void app_exit_cb		 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
 
+static void search_cb		 (MainWindow	*mainwin,
+				  guint		 action,
+				  GtkWidget	*widget);
+
 static void toggle_folder_cb	 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
@@ -329,15 +336,19 @@ static void allsel_cb		 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
 
-static void prefs_common_open_cb (MainWindow	*mainwin,
-				  guint		 action,
-				  GtkWidget	*widget);
-static void prefs_filter_open_cb (MainWindow	*mainwin,
-				  guint		 action,
-				  GtkWidget	*widget);
-static void prefs_account_open_cb(MainWindow	*mainwin,
-				  guint		 action,
-				  GtkWidget	*widget);
+static void prefs_common_open_cb	(MainWindow	*mainwin,
+					 guint		 action,
+					 GtkWidget	*widget);
+static void prefs_filter_open_cb	(MainWindow	*mainwin,
+					 guint		 action,
+					 GtkWidget	*widget);
+static void prefs_template_open_cb	(MainWindow	*mainwin,
+					 guint		 action,
+					 GtkWidget	*widget);
+static void prefs_account_open_cb	(MainWindow	*mainwin,
+					 guint		 action,
+					 GtkWidget	*widget);
+
 static void new_account_cb	 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
@@ -357,7 +368,7 @@ static GtkItemFactoryEntry mainwin_entries[] =
 {
 	{N_("/_File"),				NULL, NULL, 0, "<Branch>"},
 	{N_("/_File/_Add mailbox..."),		NULL, add_mailbox_cb, 0, NULL},
-	{N_("/_File/_Update folder tree"),	NULL, update_folderview_cb, 0, NULL},
+	{N_("/_File/_Rescan folder tree"),	NULL, update_folderview_cb, 0, NULL},
 	{N_("/_File/_Folder"),			NULL, NULL, 0, "<Branch>"},
 	{N_("/_File/_Folder/Create _new folder..."),
 						NULL, new_folder_cb, 0, NULL},
@@ -377,7 +388,8 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Edit/_Copy"),			"<control>C", copy_cb, 0, NULL},
 	{N_("/_Edit/Select _all"),		"<control>A", allsel_cb, 0, NULL},
 	{N_("/_Edit/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Edit/_Search"),			"<control>S", summary_search_cb, 0, NULL},
+	{N_("/_Edit/_Find in current message"),	"<control>F", search_cb, 0, NULL},
+	{N_("/_Edit/_Search folder"),		"<control>S", search_cb, 1, NULL},
 
 	{N_("/_View"),				NULL, NULL, 0, "<Branch>"},
 	{N_("/_View/_Folder tree"),		NULL, toggle_folder_cb, 0, "<ToggleItem>"},
@@ -481,7 +493,7 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Message/_Reply"),		"<alt>R", 	reply_cb, COMPOSE_REPLY, NULL},
 	{N_("/_Message/Repl_y to sender"),	"<control><alt>R", reply_cb, COMPOSE_REPLY_TO_SENDER, NULL},
 	{N_("/_Message/Reply to a_ll"),		"<shift><alt>R", reply_cb, COMPOSE_REPLY_TO_ALL, NULL},
-	{N_("/_Message/_Forward"),		"<control>F", reply_cb, COMPOSE_FORWARD, NULL},
+	{N_("/_Message/_Forward"),		"<shift><alt>F", reply_cb, COMPOSE_FORWARD, NULL},
 	{N_("/_Message/Forward as a_ttachment"),
 						"<shift><control>F", reply_cb, COMPOSE_FORWARD_AS_ATTACH, NULL},
 	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
@@ -560,7 +572,8 @@ static GtkItemFactoryEntry mainwin_entries[] =
 						NULL, prefs_common_open_cb, 0, NULL},
 	{N_("/_Configuration/_Filter setting..."),
 						NULL, prefs_filter_open_cb, 0, NULL},
-	{N_("/_Configuration/_Preferences per account..."),
+	{N_("/_Configuration/_Template..."),	NULL, prefs_template_open_cb, 0, NULL},
+	{N_("/_Configuration/_Preferences for current account..."),
 						NULL, prefs_account_open_cb, 0, NULL},
 	{N_("/_Configuration/---"),		NULL, NULL, 0, "<Separator>"},
 	{N_("/_Configuration/Create _new account..."),
@@ -865,13 +878,23 @@ void main_window_reflect_prefs_all(void)
 		else
 			gtk_widget_show(mainwin->exec_btn);
 
-		summary_change_display_item(mainwin->summaryview);
 		summary_redisplay_msg(mainwin->summaryview);
 		headerview_set_visibility(mainwin->messageview->headerview,
 					  prefs_common.display_header_pane);
 	}
 }
 
+void main_window_set_summary_column(void)
+{
+	GList *cur;
+	MainWindow *mainwin;
+
+	for (cur = mainwin_list; cur != NULL; cur = cur->next) {
+		mainwin = (MainWindow *)cur->data;
+		summary_set_column_order(mainwin->summaryview);
+	}
+}
+
 void main_window_set_account_menu(GList *account_list)
 {
 	GList *cur, *cur_ac, *cur_item;
@@ -923,11 +946,8 @@ static void main_window_show_cur_account(MainWindow *mainwin)
 	gtk_window_set_title(GTK_WINDOW(mainwin->window), buf);
 	g_free(buf);
 
-	//buf = g_strdup_printf(_("Current account: %s"), ac_name);
-	//gtk_label_set_text(GTK_LABEL(mainwin->ac_label), buf);
 	gtk_label_set_text(GTK_LABEL(mainwin->ac_label), ac_name);
 	gtk_widget_queue_resize(mainwin->ac_button);
-	//g_free(buf);
 
 	g_free(ac_name);
 }
@@ -1092,13 +1112,16 @@ typedef enum
 	M_SINGLE_TARGET_EXIST = 1 << 3,
 	M_EXEC                = 1 << 4,
 	M_ALLOW_REEDIT        = 1 << 5,
-	M_HAVE_ACCOUNT        = 1 << 6
+	M_HAVE_ACCOUNT        = 1 << 6,
+	M_THREADED	      = 1 << 7,
+	M_UNTHREADED	      = 1 << 8
 } SensitiveCond;
 
 static SensitiveCond main_window_get_current_state(MainWindow *mainwin)
 {
 	SensitiveCond state = 0;
 	SummarySelection selection;
+	FolderItem *item = mainwin->summaryview->folder_item;
 
 	selection = summary_get_selection_type(mainwin->summaryview);
 
@@ -1106,18 +1129,23 @@ static SensitiveCond main_window_get_current_state(MainWindow *mainwin)
 		state |= M_UNLOCKED;
 	if (selection != SUMMARY_NONE)
 		state |= M_MSG_EXIST;
+	if (item) {
+		if (item->threaded)
+			state |= M_THREADED;
+		else
+			state |= M_UNTHREADED;	
+	}		
 	if (selection == SUMMARY_SELECTED_SINGLE ||
 	    selection == SUMMARY_SELECTED_MULTIPLE)
 		state |= M_TARGET_EXIST;
 	if (selection == SUMMARY_SELECTED_SINGLE)
 		state |= M_SINGLE_TARGET_EXIST;
-	if (mainwin->summaryview->folder_item &&
-	    mainwin->summaryview->folder_item->folder->type != F_NEWS)
+	if (item && item->folder->type != F_NEWS)
 		state |= M_EXEC;
 	if (selection == SUMMARY_SELECTED_SINGLE &&
-	    (mainwin->summaryview->folder_item &&
-	     (mainwin->summaryview->folder_item->stype == F_DRAFT ||
-	      mainwin->summaryview->folder_item->stype == F_QUEUE)))
+	    (item &&
+	     (item->stype == F_OUTBOX || item->stype == F_DRAFT ||
+	      item->stype == F_QUEUE)))
 		state |= M_ALLOW_REEDIT;
 	if (cur_account)
 		state |= M_HAVE_ACCOUNT;
@@ -1169,7 +1197,7 @@ void main_window_set_menu_sensitive(MainWindow *mainwin)
 		SensitiveCond cond;
 	} entry[] = {
 		{"/File/Add mailbox..."        , M_UNLOCKED},
-		{"/File/Update folder tree"    , M_UNLOCKED},
+		{"/File/Rescan folder tree"    , M_UNLOCKED},
 		{"/File/Folder"                , M_UNLOCKED},
 		{"/File/Import mbox file..."   , M_UNLOCKED},
 		{"/File/Export to mbox file...", M_UNLOCKED},
@@ -1210,6 +1238,8 @@ void main_window_set_menu_sensitive(MainWindow *mainwin)
 		{"/Summary/Go to/Prev labeled message", M_MSG_EXIST},
 		{"/Summary/Go to/Next labeled message", M_MSG_EXIST},
 		{"/Summary/Sort"                      , M_MSG_EXIST},
+		{"/Summary/Thread view"               ,	M_UNTHREADED},
+		{"/Summary/Unthread view"             , M_THREADED},
 
 		{"/Configuration", M_UNLOCKED},
 
@@ -1234,8 +1264,8 @@ void main_window_popup(MainWindow *mainwin)
 	gdk_window_get_origin(mainwin->window->window, &x, &y);
 	sx = gdk_screen_width();
 	sy = gdk_screen_height();
-	x %= sx; if (x < 0) x += sx;
-	y %= sy; if (y < 0) y += sy;
+	x %= sx; if (x < 0) x = 0;
+	y %= sy; if (y < 0) y = 0;
 	gdk_window_move(mainwin->window->window, x, y);
 	gdk_window_raise(mainwin->window->window);
 	gdk_window_show(mainwin->window->window);
@@ -1246,30 +1276,30 @@ void main_window_popup(MainWindow *mainwin)
 	case SEPARATE_FOLDER:
 		widget = mainwin->win.sep_folder.folderwin;
 		gdk_window_get_origin(widget->window, &x, &y);
-		x %= sx; if (x < 0) x += sx;
-		y %= sy; if (y < 0) y += sy;
+		x %= sx; if (x < 0) x = 0;
+		y %= sy; if (y < 0) y = 0;
 		gdk_window_move(widget->window, x, y);
 		gdk_window_raise(widget->window);
 		break;
 	case SEPARATE_MESSAGE:
 		widget = mainwin->win.sep_message.messagewin;
 		gdk_window_get_origin(widget->window, &x, &y);
-		x %= sx; if (x < 0) x += sx;
-		y %= sy; if (y < 0) y += sy;
+		x %= sx; if (x < 0) x = 0;
+		y %= sy; if (y < 0) y = 0;
 		gdk_window_move(widget->window, x, y);
 		gdk_window_raise(widget->window);
 		break;
 	case SEPARATE_BOTH:
 		widget = mainwin->win.sep_both.folderwin;
 		gdk_window_get_origin(widget->window, &x, &y);
-		x %= sx; if (x < 0) x += sx;
-		y %= sy; if (y < 0) y += sy;
+		x %= sx; if (x < 0) x = 0;
+		y %= sy; if (y < 0) y = 0;
 		gdk_window_move(widget->window, x, y);
 		gdk_window_raise(widget->window);
 		widget = mainwin->win.sep_both.messagewin;
 		gdk_window_get_origin(widget->window, &x, &y);
-		x %= sx; if (x < 0) x += sx;
-		y %= sy; if (y < 0) y += sy;
+		x %= sx; if (x < 0) x = 0;
+		y %= sy; if (y < 0) y = 0;
 		gdk_window_move(widget->window, x, y);
 		gdk_window_raise(widget->window);
 		break;
@@ -1749,6 +1779,7 @@ static void ac_menu_popup_closed(GtkMenuShell *menu_shell, gpointer data)
 	GtkWidget *button;
 
 	button = gtk_object_get_data(GTK_OBJECT(menu_shell), "menu_button");
+	if (!button) return;
 	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 	gtk_object_remove_data(GTK_OBJECT(mainwin->ac_menu), "menu_button");
 	manage_window_focus_in(mainwin->window, NULL, NULL);
@@ -1836,6 +1867,14 @@ static void app_exit_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 	app_will_exit(widget, mainwin);
 }
 
+static void search_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
+{
+	if (action == 1)
+		summary_search(mainwin->summaryview);
+	else
+		message_search(mainwin->messageview);
+}
+
 static void toggle_folder_cb(MainWindow *mainwin, guint action,
 			     GtkWidget *widget)
 {
@@ -2111,16 +2150,18 @@ static void thread_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 {
 	GtkItemFactory *ifactory;
 
+	if (!mainwin->summaryview->folder_item) return;
+
 	ifactory = gtk_item_factory_from_widget(widget);
 
 	if (0 == action) {
-		summary_thread_build(mainwin->summaryview, FALSE);
-		prefs_common.enable_thread = TRUE;
+		summary_thread_build(mainwin->summaryview);
+		mainwin->summaryview->folder_item->threaded = TRUE;
 		menu_set_sensitive(ifactory, "/Summary/Thread view",   FALSE);
 		menu_set_sensitive(ifactory, "/Summary/Unthread view", TRUE);
 	} else {
 		summary_unthread(mainwin->summaryview);
-		prefs_common.enable_thread = FALSE;
+		mainwin->summaryview->folder_item->threaded = FALSE;
 		menu_set_sensitive(ifactory, "/Summary/Thread view",   TRUE);
 		menu_set_sensitive(ifactory, "/Summary/Unthread view", FALSE);
 	}
@@ -2129,7 +2170,7 @@ static void thread_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 static void set_display_item_cb(MainWindow *mainwin, guint action,
 				GtkWidget *widget)
 {
-	prefs_summary_display_item_set();
+	prefs_summary_column_open();
 }
 
 static void sort_summary_cb(MainWindow *mainwin, guint action,
@@ -2260,6 +2301,12 @@ static void prefs_filter_open_cb(MainWindow *mainwin, guint action,
 	prefs_filter_open();
 }
 
+static void prefs_template_open_cb(MainWindow *mainwin, guint action,
+				   GtkWidget *widget)
+{
+	prefs_template_open();
+}
+
 static void prefs_account_open_cb(MainWindow *mainwin, guint action,
 				  GtkWidget *widget)
 {
