@@ -72,7 +72,7 @@ static void mimeview_show_message_part		(MimeView	*mimeview,
 static void mimeview_show_image_part		(MimeView	*mimeview,
 						 MimeInfo	*partinfo);
 static void mimeview_change_view_type		(MimeView	*mimeview,
-						 ContentType	 type);
+						 MimeViewType	 type);
 static void mimeview_clear			(MimeView	*mimeview);
 
 static void mimeview_selected		(GtkCTree	*ctree,
@@ -269,7 +269,7 @@ void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
 	}
 	mimeview->file = g_strdup(file);
 
-	/* skip headers */
+	/* skip MIME part headers */
 	if (mimeinfo->mime_type == MIME_MULTIPART) {
 		if (fseek(fp, mimeinfo->fpos, SEEK_SET) < 0)
 		perror("fseek");
@@ -294,8 +294,21 @@ void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
 	gtk_signal_handler_unblock_by_func(GTK_OBJECT(ctree),
 					   mimeview_selected, mimeview);
 
-	if ((node = gtk_ctree_node_nth(ctree, 0))) {
+	/* search first text part */
+	for (node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
+	     node != NULL; node = GTK_CTREE_NODE_NEXT(node)) {
+		MimeInfo *partinfo;
+
+		partinfo = gtk_ctree_node_get_row_data(ctree, node);
+		if (partinfo && partinfo->mime_type == MIME_TEXT)
+			break;
+	}
+	if (!node)
+		node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
+
+	if (node) {
 		gtk_ctree_select(ctree, node);
+		gtkut_ctree_set_focus_row(ctree, node);
 		gtk_widget_grab_focus(mimeview->ctree);
 	}
 }
@@ -608,7 +621,6 @@ static void mimeview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 		}
 		break;
 	case GDK_BackSpace:
-	case GDK_Delete:
 		textview_scroll_page(mimeview->textview, TRUE);
 		return;
 	case GDK_Return:
@@ -756,6 +768,7 @@ static void mimeview_open_with(MimeView *mimeview)
 {
 	MimeInfo *partinfo;
 	gchar *filename;
+	gchar *cmd;
 
 	if (!mimeview->opened) return;
 	if (!mimeview->file) return;
@@ -766,21 +779,29 @@ static void mimeview_open_with(MimeView *mimeview)
 
 	filename = procmime_get_tmp_file_name(partinfo);
 
-	if (procmime_get_part(filename, mimeview->file, partinfo) < 0)
+	if (procmime_get_part(filename, mimeview->file, partinfo) < 0) {
 		alertpanel_error
 			(_("Can't save the part of multipart message."));
-	else {
-		gchar *cmd;
+		g_free(filename);
+		return;
+	}
 
-		cmd = input_dialog
-			(_("Open with"),
-			 _("Enter the command line to open file:\n"
-			   "(`%s' will be replaced with file name)"),
-			 "gedit '%s'");
-		if (cmd) {
-			mimeview_view_file(filename, partinfo, cmd);
-			g_free(cmd);
-		}
+	if (!prefs_common.mime_open_cmd_history)
+		prefs_common.mime_open_cmd_history =
+			add_history(NULL, prefs_common.mime_open_cmd);
+
+	cmd = input_dialog_combo
+		(_("Open with"),
+		 _("Enter the command line to open file:\n"
+		   "(`%s' will be replaced with file name)"),
+		 prefs_common.mime_open_cmd,
+		 prefs_common.mime_open_cmd_history);
+	if (cmd) {
+		mimeview_view_file(filename, partinfo, cmd);
+		g_free(prefs_common.mime_open_cmd);
+		prefs_common.mime_open_cmd = cmd;
+		prefs_common.mime_open_cmd_history =
+			add_history(prefs_common.mime_open_cmd_history, cmd);
 	}
 
 	g_free(filename);
