@@ -71,10 +71,16 @@ gint pop3_getauth_user_send(SockInfo *sock, gpointer data)
 
 gint pop3_getauth_user_recv(SockInfo *sock, gpointer data)
 {
+	Pop3State *state = (Pop3State *)data;
+
 	if (pop3_ok(sock, NULL) == PS_SUCCESS)
 		return POP3_GETAUTH_PASS_SEND;
-	else
+	else {
+		log_warning(_("error occurred on authentication\n"));
+		state->error_val = PS_AUTHFAIL;
+		state->inc_state = INC_AUTH_FAILED;
 		return -1;
+	}
 }
 
 gint pop3_getauth_pass_send(SockInfo *sock, gpointer data)
@@ -383,7 +389,7 @@ gint pop3_delete_send(SockInfo *sock, gpointer data)
 {
 	Pop3State *state = (Pop3State *)data;
 
-	//inc_progress_update(state, POP3_DELETE_SEND);
+	/* inc_progress_update(state, POP3_DELETE_SEND); */
 
 	pop3_gen_send(sock, "DELE %d", state->cur_msg);
 
