@@ -32,12 +32,14 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <errno.h>
 
 #include "intl.h"
 #include "main.h"
 #include "prefs.h"
 #include "prefs_common.h"
 #include "prefs_display_header.h"
+#include "prefs_summary_column.h"
 #include "mainwindow.h"
 #include "summaryview.h"
 #include "messageview.h"
@@ -86,21 +88,28 @@ static struct Send {
 } send;
 
 static struct Compose {
-	GtkWidget *checkbtn_quote;
-	GtkWidget *entry_quotemark;
-	GtkWidget *text_quotefmt;
 	GtkWidget *checkbtn_autosig;
 	GtkWidget *entry_sigsep;
 
 	GtkWidget *checkbtn_autoextedit;
-	GtkWidget *checkbtn_reply_account_autosel;
 
 	GtkWidget *spinbtn_linewrap;
 	GtkObject *spinbtn_linewrap_adj;
 	GtkWidget *checkbtn_wrapquote;
 	GtkWidget *checkbtn_wrapatsend;
+
+	GtkWidget *checkbtn_reply_account_autosel;
+	GtkWidget *checkbtn_quote;
 } compose;
 
+static struct Quote {
+	GtkWidget *entry_quotemark;
+	GtkWidget *text_quotefmt;
+
+	GtkWidget *entry_fw_quotemark;
+	GtkWidget *text_fw_quotefmt;
+} quote;
+
 static struct Display {
 	GtkWidget *entry_textfont;
 	GtkWidget *button_textfont;
@@ -249,15 +258,6 @@ static PrefParam param[] = {
 	 prefs_common_charset_set_optmenu},
 
 	/* Compose */
-	{"reply_with_quote", "TRUE", &prefs_common.reply_with_quote, P_BOOL,
-	 &compose.checkbtn_quote,
-	 prefs_set_data_from_toggle, prefs_set_toggle},
-	{"quote_mark", "> ", &prefs_common.quotemark, P_STRING,
-	 &compose.entry_quotemark, prefs_set_data_from_entry, prefs_set_entry},
-	{"quote_format", "On %d\\n%f wrote:\\n\\n",
-	 &prefs_common.quotefmt, P_STRING, &compose.text_quotefmt,
-	 prefs_set_data_from_text, prefs_set_text},
-
 	{"auto_signature", "TRUE", &prefs_common.auto_sig, P_BOOL,
 	 &compose.checkbtn_autosig,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
@@ -267,10 +267,6 @@ static PrefParam param[] = {
 	{"auto_ext_editor", "FALSE", &prefs_common.auto_exteditor, P_BOOL,
 	 &compose.checkbtn_autoextedit,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
-	{"reply_account_autoselect", "TRUE",
-	 &prefs_common.reply_account_autosel, P_BOOL,
-	 &compose.checkbtn_reply_account_autosel,
-	 prefs_set_data_from_toggle, prefs_set_toggle},
 
 	{"linewrap_length", "74", &prefs_common.linewrap_len, P_INT,
 	 &compose.spinbtn_linewrap,
@@ -283,9 +279,34 @@ static PrefParam param[] = {
 	 &compose.checkbtn_wrapatsend,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 
+	{"reply_with_quote", "TRUE", &prefs_common.reply_with_quote, P_BOOL,
+	 &compose.checkbtn_quote,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
+	{"reply_account_autoselect", "TRUE",
+	 &prefs_common.reply_account_autosel, P_BOOL,
+	 &compose.checkbtn_reply_account_autosel,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
+
 	{"show_ruler", "TRUE", &prefs_common.show_ruler, P_BOOL,
 	 NULL, NULL, NULL},
 
+	/* Quote */
+	{"reply_quote_mark", "> ", &prefs_common.quotemark, P_STRING,
+	 &quote.entry_quotemark, prefs_set_data_from_entry, prefs_set_entry},
+	{"reply_quote_format", "On %d\\n%f wrote:\\n\\n%Q",
+	 &prefs_common.quotefmt, P_STRING, &quote.text_quotefmt,
+	 prefs_set_data_from_text, prefs_set_text},
+
+	{"fw_quote_mark", "> ", &prefs_common.fw_quotemark, P_STRING,
+	 &quote.entry_fw_quotemark,
+	 prefs_set_data_from_entry, prefs_set_entry},
+	{"fw_quote_format",
+	 "\\n\\nBegin forwarded message:\\n\\n"
+	 "?d(Date: %d\\n)?f(From: %f\\n)?t(To: %t\\n)?c(Cc: %c\\n)"
+	 "?n(Newsgroups: %n\\n)?s(Subject: %s\\n)\\n\\n%M",
+	 &prefs_common.fw_quotefmt, P_STRING, &quote.text_fw_quotefmt,
+	 prefs_set_data_from_text, prefs_set_text},
+
 	/* Display */
 	{"widget_font", NULL, &prefs_common.widgetfont, P_STRING,
 	 NULL, NULL, NULL},
@@ -333,22 +354,56 @@ static PrefParam param[] = {
 	{"show_statusbar", "TRUE", &prefs_common.show_statusbar, P_BOOL,
 	 NULL, NULL, NULL},
 
-	{"show_mark", "TRUE", &prefs_common.show_mark, P_BOOL,
-	 NULL, NULL, NULL},
-	{"show_unread", "TRUE", &prefs_common.show_unread, P_BOOL,
-	 NULL, NULL, NULL},
-	{"show_mime", "TRUE", &prefs_common.show_mime, P_BOOL,
-	 NULL, NULL, NULL},
-	{"show_number", "FALSE", &prefs_common.show_number, P_BOOL,
-	 NULL, NULL, NULL},
-	{"show_size", "FALSE", &prefs_common.show_size, P_BOOL,
-	 NULL, NULL, NULL},
-	{"show_date", "TRUE", &prefs_common.show_date, P_BOOL,
-	 NULL, NULL, NULL},
-	{"show_from", "TRUE", &prefs_common.show_from, P_BOOL,
-	 NULL, NULL, NULL},
-	{"show_subject", "TRUE", &prefs_common.show_subject, P_BOOL,
-	 NULL, NULL, NULL},
+	{"summary_col_show_mark", "TRUE",
+	 &prefs_common.summary_col_visible[S_COL_MARK], P_BOOL, NULL, NULL, NULL},
+	{"summary_col_show_unread", "TRUE",
+	 &prefs_common.summary_col_visible[S_COL_UNREAD], P_BOOL, NULL, NULL, NULL},
+	{"summary_col_show_mime", "TRUE",
+	 &prefs_common.summary_col_visible[S_COL_MIME], P_BOOL, NULL, NULL, NULL},
+	{"summary_col_show_subject", "TRUE",
+	 &prefs_common.summary_col_visible[S_COL_SUBJECT], P_BOOL, NULL, NULL, NULL},
+	{"summary_col_show_from", "TRUE",
+	 &prefs_common.summary_col_visible[S_COL_FROM], P_BOOL, NULL, NULL, NULL},
+	{"summary_col_show_date", "TRUE",
+	 &prefs_common.summary_col_visible[S_COL_DATE], P_BOOL, NULL, NULL, NULL},
+	{"summary_col_show_size", "TRUE",
+	 &prefs_common.summary_col_visible[S_COL_SIZE], P_BOOL, NULL, NULL, NULL},
+	{"summary_col_show_number", "FALSE",
+	 &prefs_common.summary_col_visible[S_COL_NUMBER], P_BOOL, NULL, NULL, NULL},
+
+	{"summary_col_pos_mark", "0",
+	  &prefs_common.summary_col_pos[S_COL_MARK], P_INT, NULL, NULL, NULL},
+	{"summary_col_pos_unread", "1",
+	  &prefs_common.summary_col_pos[S_COL_UNREAD], P_INT, NULL, NULL, NULL},
+	{"summary_col_pos_mime", "2",
+	  &prefs_common.summary_col_pos[S_COL_MIME], P_INT, NULL, NULL, NULL},
+	{"summary_col_pos_subject", "3",
+	  &prefs_common.summary_col_pos[S_COL_SUBJECT], P_INT, NULL, NULL, NULL},
+	{"summary_col_pos_from", "4",
+	  &prefs_common.summary_col_pos[S_COL_FROM], P_INT, NULL, NULL, NULL},
+	{"summary_col_pos_date", "5",
+	  &prefs_common.summary_col_pos[S_COL_DATE], P_INT, NULL, NULL, NULL},
+	{"summary_col_pos_size", "6",
+	  &prefs_common.summary_col_pos[S_COL_SIZE], P_INT, NULL, NULL, NULL},
+	{"summary_col_pos_number", "7",
+	  &prefs_common.summary_col_pos[S_COL_NUMBER], P_INT, NULL, NULL, NULL},
+
+	{"summary_col_size_mark", "10",
+	 &prefs_common.summary_col_size[S_COL_MARK], P_INT, NULL, NULL, NULL},
+	{"summary_col_size_unread", "13",
+	 &prefs_common.summary_col_size[S_COL_UNREAD], P_INT, NULL, NULL, NULL},
+	{"summary_col_size_mime", "10",
+	 &prefs_common.summary_col_size[S_COL_MIME], P_INT, NULL, NULL, NULL},
+	{"summary_col_size_subject", "200",
+	 &prefs_common.summary_col_size[S_COL_SUBJECT], P_INT, NULL, NULL, NULL},
+	{"summary_col_size_from", "120",
+	 &prefs_common.summary_col_size[S_COL_FROM], P_INT, NULL, NULL, NULL},
+	{"summary_col_size_date", "118",
+	 &prefs_common.summary_col_size[S_COL_DATE], P_INT, NULL, NULL, NULL},
+	{"summary_col_size_size", "45",
+	 &prefs_common.summary_col_size[S_COL_SIZE], P_INT, NULL, NULL, NULL},
+	{"summary_col_size_number", "40",
+	 &prefs_common.summary_col_size[S_COL_NUMBER], P_INT, NULL, NULL, NULL},
 
 	/* Widget size */
 	{"folderview_width", "179", &prefs_common.folderview_width, P_INT,
@@ -363,26 +418,12 @@ static PrefParam param[] = {
 	 NULL, NULL, NULL},
 	{"folder_col_total", "32", &prefs_common.folder_col_total, P_INT,
 	 NULL, NULL, NULL},
+
 	{"summaryview_width", "600", &prefs_common.summaryview_width, P_INT,
 	 NULL, NULL, NULL},
 	{"summaryview_height", "173", &prefs_common.summaryview_height, P_INT,
 	 NULL, NULL, NULL},
-	{"summary_col_mark", "10", &prefs_common.summary_col_mark, P_INT,
-	 NULL, NULL, NULL},
-	{"summary_col_unread", "13", &prefs_common.summary_col_unread, P_INT,
-	 NULL, NULL, NULL},
-	{"summary_col_mime", "10", &prefs_common.summary_col_mime, P_INT,
-	 NULL, NULL, NULL},
-	{"summary_col_number", "40", &prefs_common.summary_col_number, P_INT,
-	 NULL, NULL, NULL},
-	{"summary_col_size", "48", &prefs_common.summary_col_size, P_INT,
-	 NULL, NULL, NULL},
-	{"summary_col_date", "120", &prefs_common.summary_col_date, P_INT,
-	 NULL, NULL, NULL},
-	{"summary_col_from", "140", &prefs_common.summary_col_from, P_INT,
-	 NULL, NULL, NULL},
-	{"summary_col_subject", "200", &prefs_common.summary_col_subject,
-	 P_INT, NULL, NULL, NULL},
+
 	{"mainview_x", "64", &prefs_common.mainview_x, P_INT,
 	 NULL, NULL, NULL},
 	{"mainview_y", "64", &prefs_common.mainview_y, P_INT,
@@ -461,6 +502,8 @@ static PrefParam param[] = {
 	 &prefs_common.mime_image_viewer, P_STRING, NULL, NULL, NULL},
 	{"mime_audio_player", "play '%s'",
 	 &prefs_common.mime_audio_player, P_STRING, NULL, NULL, NULL},
+	{"mime_open_command", "gedit '%s'",
+	 &prefs_common.mime_open_cmd, P_STRING, NULL, NULL, NULL},
 
 #if USE_GPGME
 	/* Privacy */
@@ -549,6 +592,7 @@ static void prefs_common_create		(void);
 static void prefs_receive_create	(void);
 static void prefs_send_create		(void);
 static void prefs_compose_create	(void);
+static void prefs_quote_create		(void);
 static void prefs_display_create	(void);
 static void prefs_message_create	(void);
 #if USE_GPGME
@@ -577,8 +621,11 @@ static void date_format_select_row		(GtkWidget	*date_format_list,
 static GtkWidget *date_format_create		(GtkButton	*button,
 						 void		*data);
 
-static void prefs_quote_description		(void);
 static void prefs_quote_description_create	(void);
+static void prefs_quote_description_key_pressed	(GtkWidget	*widget,
+						 GdkEventKey	*event,
+						 gpointer	 data);
+
 static void prefs_quote_colors_dialog		(void);
 static void prefs_quote_colors_dialog_create	(void);
 static void prefs_quote_colors_key_pressed	(GtkWidget	*widget,
@@ -598,20 +645,6 @@ static void set_button_bg_color			(GtkWidget	*widget,
 static void prefs_enable_message_color_toggled	(void);
 static void prefs_recycle_colors_toggled	(GtkWidget	*widget);
 
-/* functions for setting items of SummaryView */
-static void prefs_summary_display_item_dialog_create
-					(gboolean	*cancelled);
-static void display_item_ok		(GtkWidget	*widget,
-					 gboolean	*cancelled);
-static void display_item_cancel		(GtkWidget	*widget,
-					 gboolean	*cancelled);
-static gint display_item_delete_event	(GtkWidget	*widget,
-					 GdkEventAny	*event,
-					 gboolean	*cancelled);
-static void display_item_key_pressed	(GtkWidget	*widget,
-					 GdkEventKey	*event,
-					 gboolean	*cancelled);
-
 static void prefs_font_select			(GtkButton	*button);
 static void prefs_font_selection_key_pressed	(GtkWidget	*widget,
 						 GdkEventKey	*event,
@@ -630,12 +663,56 @@ static void prefs_common_cancel		(void);
 
 void prefs_common_read_config(void)
 {
+	FILE *fp;
+	gchar *path;
+	gchar buf[PREFSBUFSIZE];
+
 	prefs_read_config(param, "Common", COMMON_RC);
+
+	path = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, COMMAND_HISTORY,
+			   NULL);
+	if ((fp = fopen(path, "r")) == NULL) {
+		if (ENOENT != errno) FILE_OP_ERROR(path, "fopen");
+		g_free(path);
+		return;
+	}
+	g_free(path);
+	while (fgets(buf, sizeof(buf), fp) != NULL) {
+		g_strstrip(buf);
+		if (buf[0] == '\0') continue;
+		prefs_common.mime_open_cmd_history =
+			add_history(prefs_common.mime_open_cmd_history, buf);
+	}
+	fclose(fp);
+
+	prefs_common.mime_open_cmd_history =
+		g_list_reverse(prefs_common.mime_open_cmd_history);
 }
 
 void prefs_common_save_config(void)
 {
+	GList *cur;
+	FILE *fp;
+	gchar *path;
+
 	prefs_save_config(param, "Common", COMMON_RC);
+
+	path = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, COMMAND_HISTORY,
+			   NULL);
+	if ((fp = fopen(path, "w")) == NULL) {
+		FILE_OP_ERROR(path, "fopen");
+		g_free(path);
+		return;
+	}
+
+	for (cur = prefs_common.mime_open_cmd_history;
+	     cur != NULL; cur = cur->next) {
+		fputs((gchar *)cur->data, fp);
+		fputc('\n', fp);
+	}
+
+	fclose(fp);
+	g_free(path);
 }
 
 void prefs_common_open(void)
@@ -686,6 +763,8 @@ static void prefs_common_create(void)
 	SET_NOTEBOOK_LABEL(dialog.notebook, _("Send"),      page++);
 	prefs_compose_create();
 	SET_NOTEBOOK_LABEL(dialog.notebook, _("Compose"),   page++);
+	prefs_quote_create();
+	SET_NOTEBOOK_LABEL(dialog.notebook, _("Quote"),   page++);
 	prefs_display_create();
 	SET_NOTEBOOK_LABEL(dialog.notebook, _("Display"),   page++);
 	prefs_message_create();
@@ -754,7 +833,7 @@ static void prefs_receive_create(void)
 	gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 0);
 	SET_TOGGLE_SENSITIVITY (checkbtn_incext, hbox);
 
-	label_incext = gtk_label_new (_("Program path"));
+	label_incext = gtk_label_new (_("Command"));
 	gtk_widget_show (label_incext);
 	gtk_box_pack_start (GTK_BOX (hbox), label_incext, FALSE, FALSE, 0);
 
@@ -809,7 +888,7 @@ static void prefs_receive_create(void)
 	PACK_CHECK_BUTTON (hbox_autochk, checkbtn_autochk,
 			   _("Auto-check new mail"));
 
-	label_autochk1 = gtk_label_new (_("each"));
+	label_autochk1 = gtk_label_new (_("every"));
 	gtk_widget_show (label_autochk1);
 	gtk_box_pack_start (GTK_BOX (hbox_autochk), label_autochk1, FALSE, FALSE, 0);
 
@@ -842,7 +921,7 @@ static void prefs_receive_create(void)
 	gtk_container_set_border_width (GTK_CONTAINER (hbox), 8);
 
 	label_maxarticle = gtk_label_new
-		(_("Maximum article number to download\n"
+		(_("Maximum number of articles to download\n"
 		   "(unlimited if 0 is specified)"));
 	gtk_widget_show (label_maxarticle);
 	gtk_box_pack_start (GTK_BOX (hbox), label_maxarticle, FALSE, FALSE, 0);
@@ -895,6 +974,7 @@ static void prefs_send_create(void)
 	GtkWidget *optmenu;
 	GtkWidget *optmenu_menu;
 	GtkWidget *menuitem;
+	GtkWidget *label_charset_desc;
 
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
@@ -916,7 +996,7 @@ static void prefs_send_create(void)
 	gtk_box_pack_start (GTK_BOX (vbox_extsend), hbox1, FALSE, FALSE, 0);
 	SET_TOGGLE_SENSITIVITY(checkbtn_extsend, hbox1);
 
-	label_extsend = gtk_label_new (_("Program path"));
+	label_extsend = gtk_label_new (_("Command"));
 	gtk_widget_show (label_extsend);
 	gtk_box_pack_start (GTK_BOX (hbox1), label_extsend, FALSE, FALSE, 0);
 
@@ -949,7 +1029,7 @@ static void prefs_send_create(void)
 
 	optmenu = gtk_option_menu_new ();
 	gtk_widget_show (optmenu);
-	gtk_box_pack_start(GTK_BOX (hbox1), optmenu, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox1), optmenu, FALSE, FALSE, 0);
 
 	optmenu_menu = gtk_menu_new ();
 
@@ -992,6 +1072,18 @@ static void prefs_send_create(void)
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (optmenu), optmenu_menu);
 
+	hbox1 = gtk_hbox_new (FALSE, 8);
+	gtk_widget_show (hbox1);
+	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);
+
+	label_charset_desc = gtk_label_new
+		(_("If `Automatic' is selected, the optimal encoding\n"
+		   "for the current locale will be used."));
+	gtk_widget_show (label_charset_desc);
+	gtk_box_pack_start (GTK_BOX (hbox1), label_charset_desc,
+			    FALSE, FALSE, 0);
+	gtk_label_set_justify(GTK_LABEL (label_charset_desc), GTK_JUSTIFY_LEFT);
+
 	send.checkbtn_extsend = checkbtn_extsend;
 	send.entry_extsend    = entry_extsend;
 	/* send.button_extsend   = button_extsend; */
@@ -1005,26 +1097,16 @@ static void prefs_send_create(void)
 static void prefs_compose_create(void)
 {
 	GtkWidget *vbox1;
-	GtkWidget *frame_quote;
-	GtkWidget *vbox_quote;
-	GtkWidget *checkbtn_quote;
 	GtkWidget *hbox1;
-	GtkWidget *label_quotemark;
-	GtkWidget *entry_quotemark;
 	GtkWidget *hbox2;
-	GtkWidget *label_quotefmt;
-	GtkWidget *btn_quotedesc;
-	GtkWidget *scrolledwin_quotefmt;
-	GtkWidget *text_quotefmt;
 
 	GtkWidget *frame_sig;
 	GtkWidget *checkbtn_autosig;
 	GtkWidget *label_sigsep;
 	GtkWidget *entry_sigsep;
 
+	GtkWidget *frame_editor;
 	GtkWidget *checkbtn_autoextedit;
-	GtkWidget *checkbtn_reply_account_autosel;
-
 	GtkWidget *vbox2;
 	GtkWidget *vbox3;
 	GtkWidget *hbox3;
@@ -1035,66 +1117,15 @@ static void prefs_compose_create(void)
 	GtkWidget *checkbtn_wrapquote;
 	GtkWidget *checkbtn_wrapatsend;
 
+	GtkWidget *frame_reply;
+	GtkWidget *checkbtn_reply_account_autosel;
+	GtkWidget *checkbtn_quote;
+
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
 	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);
 
-	PACK_FRAME(vbox1, frame_quote, _("Quotation"));
-
-	vbox_quote = gtk_vbox_new (FALSE, VSPACING_NARROW);
-	gtk_widget_show (vbox_quote);
-	gtk_container_add (GTK_CONTAINER (frame_quote), vbox_quote);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox_quote), 8);
-
-	hbox1 = gtk_hbox_new (FALSE, 32);
-	gtk_widget_show (hbox1);
-	gtk_box_pack_start (GTK_BOX (vbox_quote), hbox1, FALSE, FALSE, 0);
-
-	hbox2 = gtk_hbox_new (FALSE, 8);
-	gtk_widget_show (hbox2);
-	gtk_box_pack_start (GTK_BOX (hbox1), hbox2, FALSE, FALSE, 0);
-
-	label_quotemark = gtk_label_new (_("Quotation mark"));
-	gtk_widget_show (label_quotemark);
-	gtk_box_pack_start (GTK_BOX (hbox2), label_quotemark, FALSE, FALSE, 0);
-
-	entry_quotemark = gtk_entry_new ();
-	gtk_widget_show (entry_quotemark);
-	gtk_box_pack_start (GTK_BOX (hbox2), entry_quotemark, FALSE, FALSE, 0);
-	gtk_widget_set_usize (entry_quotemark, 64, -1);
-
-	PACK_CHECK_BUTTON (hbox1, checkbtn_quote,
-			   _("Quote message when replying"));
-
-	hbox2 = gtk_hbox_new (FALSE, 0);
-	gtk_widget_show (hbox2);
-	gtk_box_pack_start (GTK_BOX (vbox_quote), hbox2, TRUE, TRUE, 0);
-
-	label_quotefmt = gtk_label_new (_("Quotation format:"));
-	gtk_widget_show (label_quotefmt);
-	gtk_box_pack_start (GTK_BOX (hbox2), label_quotefmt, FALSE, FALSE, 0);
-
-	btn_quotedesc =
-		gtk_button_new_with_label (_(" Description of symbols "));
-	gtk_widget_show (btn_quotedesc);
-	gtk_box_pack_end (GTK_BOX (hbox2), btn_quotedesc, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(btn_quotedesc), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_quote_description), NULL);
-
-	scrolledwin_quotefmt = gtk_scrolled_window_new (NULL, NULL);
-	gtk_widget_show (scrolledwin_quotefmt);
-	gtk_box_pack_start (GTK_BOX (vbox_quote), scrolledwin_quotefmt, TRUE, TRUE, 0);
-	gtk_scrolled_window_set_policy
-		(GTK_SCROLLED_WINDOW (scrolledwin_quotefmt),
-		 GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
-
-	text_quotefmt = gtk_text_new (NULL, NULL);
-	gtk_widget_show (text_quotefmt);
-	gtk_container_add(GTK_CONTAINER(scrolledwin_quotefmt), text_quotefmt);
-	gtk_text_set_editable (GTK_TEXT (text_quotefmt), TRUE);
-	gtk_widget_set_usize(text_quotefmt, -1, 60);
-
 	/* signature */
 
 	PACK_FRAME(vbox1, frame_sig, _("Signature"));
@@ -1119,14 +1150,15 @@ static void prefs_compose_create(void)
 
 	PACK_CHECK_BUTTON (hbox1, checkbtn_autosig, _("Insert automatically"));
 
+	PACK_FRAME (vbox1, frame_editor, _("Editor"));
+
 	vbox2 = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox2);
-	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (frame_editor), vbox2);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);
 
 	PACK_CHECK_BUTTON (vbox2, checkbtn_autoextedit,
 			   _("Automatically launch the external editor"));
-	PACK_CHECK_BUTTON (vbox2, checkbtn_reply_account_autosel,
-			   _("Automatically select account for replies"));
 
 	PACK_VSPACER (vbox2, vbox3, VSPACING_NARROW_2);
 
@@ -1162,19 +1194,150 @@ static void prefs_compose_create(void)
 	PACK_CHECK_BUTTON
 		(hbox4, checkbtn_wrapatsend, _("Wrap before sending"));
 
-	compose.checkbtn_quote   = checkbtn_quote;
-	compose.entry_quotemark  = entry_quotemark;
-	compose.text_quotefmt    = text_quotefmt;
+	PACK_FRAME(vbox1, frame_reply, _("Reply"));
+
+	vbox2 = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vbox2);
+	gtk_container_add (GTK_CONTAINER (frame_reply), vbox2);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);
+
+	PACK_CHECK_BUTTON (vbox2, checkbtn_reply_account_autosel,
+			   _("Automatically select account for replies"));
+	PACK_CHECK_BUTTON (vbox2, checkbtn_quote,
+			   _("Quote message when replying"));
+
 	compose.checkbtn_autosig = checkbtn_autosig;
 	compose.entry_sigsep     = entry_sigsep;
 
 	compose.checkbtn_autoextedit = checkbtn_autoextedit;
-	compose.checkbtn_reply_account_autosel = checkbtn_reply_account_autosel;
 
 	compose.spinbtn_linewrap     = spinbtn_linewrap;
 	compose.spinbtn_linewrap_adj = spinbtn_linewrap_adj;
 	compose.checkbtn_wrapquote   = checkbtn_wrapquote;
 	compose.checkbtn_wrapatsend  = checkbtn_wrapatsend;
+
+	compose.checkbtn_quote = checkbtn_quote;
+	compose.checkbtn_reply_account_autosel =
+		checkbtn_reply_account_autosel;
+}
+
+static void prefs_quote_create(void)
+{
+	GtkWidget *vbox1;
+	GtkWidget *frame_quote;
+	GtkWidget *vbox_quote;
+	GtkWidget *hbox1;
+	GtkWidget *hbox2;
+	GtkWidget *label_quotemark;
+	GtkWidget *entry_quotemark;
+	GtkWidget *scrolledwin_quotefmt;
+	GtkWidget *text_quotefmt;
+
+	GtkWidget *entry_fw_quotemark;
+	GtkWidget *text_fw_quotefmt;
+
+	GtkWidget *btn_quotedesc;
+
+	vbox1 = gtk_vbox_new (FALSE, VSPACING);
+	gtk_widget_show (vbox1);
+	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);
+
+	/* reply */
+
+	PACK_FRAME (vbox1, frame_quote, _("Reply format"));
+
+	vbox_quote = gtk_vbox_new (FALSE, VSPACING_NARROW);
+	gtk_widget_show (vbox_quote);
+	gtk_container_add (GTK_CONTAINER (frame_quote), vbox_quote);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox_quote), 8);
+
+	hbox1 = gtk_hbox_new (FALSE, 32);
+	gtk_widget_show (hbox1);
+	gtk_box_pack_start (GTK_BOX (vbox_quote), hbox1, FALSE, FALSE, 0);
+
+	hbox2 = gtk_hbox_new (FALSE, 8);
+	gtk_widget_show (hbox2);
+	gtk_box_pack_start (GTK_BOX (hbox1), hbox2, FALSE, FALSE, 0);
+
+	label_quotemark = gtk_label_new (_("Quotation mark"));
+	gtk_widget_show (label_quotemark);
+	gtk_box_pack_start (GTK_BOX (hbox2), label_quotemark, FALSE, FALSE, 0);
+
+	entry_quotemark = gtk_entry_new ();
+	gtk_widget_show (entry_quotemark);
+	gtk_box_pack_start (GTK_BOX (hbox2), entry_quotemark, FALSE, FALSE, 0);
+	gtk_widget_set_usize (entry_quotemark, 64, -1);
+
+	scrolledwin_quotefmt = gtk_scrolled_window_new (NULL, NULL);
+	gtk_widget_show (scrolledwin_quotefmt);
+	gtk_box_pack_start (GTK_BOX (vbox_quote), scrolledwin_quotefmt, TRUE, TRUE, 0);
+	gtk_scrolled_window_set_policy
+		(GTK_SCROLLED_WINDOW (scrolledwin_quotefmt),
+		 GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+
+	text_quotefmt = gtk_text_new (NULL, NULL);
+	gtk_widget_show (text_quotefmt);
+	gtk_container_add(GTK_CONTAINER(scrolledwin_quotefmt), text_quotefmt);
+	gtk_text_set_editable (GTK_TEXT (text_quotefmt), TRUE);
+	gtk_widget_set_usize(text_quotefmt, -1, 60);
+
+	/* forward */
+
+	PACK_FRAME (vbox1, frame_quote, _("Forward format"));
+
+	vbox_quote = gtk_vbox_new (FALSE, VSPACING_NARROW);
+	gtk_widget_show (vbox_quote);
+	gtk_container_add (GTK_CONTAINER (frame_quote), vbox_quote);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox_quote), 8);
+
+	hbox1 = gtk_hbox_new (FALSE, 32);
+	gtk_widget_show (hbox1);
+	gtk_box_pack_start (GTK_BOX (vbox_quote), hbox1, FALSE, FALSE, 0);
+
+	hbox2 = gtk_hbox_new (FALSE, 8);
+	gtk_widget_show (hbox2);
+	gtk_box_pack_start (GTK_BOX (hbox1), hbox2, FALSE, FALSE, 0);
+
+	label_quotemark = gtk_label_new (_("Quotation mark"));
+	gtk_widget_show (label_quotemark);
+	gtk_box_pack_start (GTK_BOX (hbox2), label_quotemark, FALSE, FALSE, 0);
+
+	entry_fw_quotemark = gtk_entry_new ();
+	gtk_widget_show (entry_fw_quotemark);
+	gtk_box_pack_start (GTK_BOX (hbox2), entry_fw_quotemark,
+			    FALSE, FALSE, 0);
+	gtk_widget_set_usize (entry_fw_quotemark, 64, -1);
+
+	scrolledwin_quotefmt = gtk_scrolled_window_new (NULL, NULL);
+	gtk_widget_show (scrolledwin_quotefmt);
+	gtk_box_pack_start (GTK_BOX (vbox_quote), scrolledwin_quotefmt, TRUE, TRUE, 0);
+	gtk_scrolled_window_set_policy
+		(GTK_SCROLLED_WINDOW (scrolledwin_quotefmt),
+		 GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+
+	text_fw_quotefmt = gtk_text_new (NULL, NULL);
+	gtk_widget_show (text_fw_quotefmt);
+	gtk_container_add(GTK_CONTAINER(scrolledwin_quotefmt),
+			  text_fw_quotefmt);
+	gtk_text_set_editable (GTK_TEXT (text_fw_quotefmt), TRUE);
+	gtk_widget_set_usize(text_fw_quotefmt, -1, 60);
+
+	hbox1 = gtk_hbox_new (FALSE, 32);
+	gtk_widget_show (hbox1);
+	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);
+
+	btn_quotedesc =
+		gtk_button_new_with_label (_(" Description of symbols "));
+	gtk_widget_show (btn_quotedesc);
+	gtk_box_pack_start (GTK_BOX (hbox1), btn_quotedesc, FALSE, FALSE, 0);
+	gtk_signal_connect(GTK_OBJECT(btn_quotedesc), "clicked",
+			   GTK_SIGNAL_FUNC(prefs_quote_description), NULL);
+
+	quote.entry_quotemark    = entry_quotemark;
+	quote.text_quotefmt      = text_quotefmt;
+	quote.entry_fw_quotemark = entry_fw_quotemark;
+	quote.text_fw_quotefmt   = text_fw_quotefmt;
 }
 
 static void prefs_display_create(void)
@@ -1293,7 +1456,7 @@ static void prefs_display_create(void)
 	gtk_widget_show (button_dispitem);
 	gtk_box_pack_start (GTK_BOX (hbox1), button_dispitem, FALSE, TRUE, 0);
 	gtk_signal_connect (GTK_OBJECT (button_dispitem), "clicked",
-			    GTK_SIGNAL_FUNC (prefs_summary_display_item_set),
+			    GTK_SIGNAL_FUNC (prefs_summary_column_open),
 			    NULL);
 
 	display.entry_textfont	= entry_textfont;
@@ -1649,7 +1812,7 @@ static void prefs_interface_create(void)
 	gtk_box_pack_start (GTK_BOX (vbox3), hbox1, FALSE, FALSE, 0);
 
 	label = gtk_label_new
-		(_("(Messages will be just marked till execution\n"
+		(_("(Messages will be marked until execution\n"
 		   " if this is turned off)"));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 8);
@@ -2372,11 +2535,12 @@ static void prefs_recycle_colors_toggled(GtkWidget *widget)
 	prefs_common.recycle_quote_colors = is_active;
 }
 
-static void prefs_quote_description(void)
+void prefs_quote_description(void)
 {
 	if (!quote_desc_win)
 		prefs_quote_description_create();
 
+	manage_window_set_transient(GTK_WINDOW(quote_desc_win));
 	gtk_widget_show(quote_desc_win);
 	gtk_main();
 	gtk_widget_hide(quote_desc_win);
@@ -2401,20 +2565,28 @@ static void prefs_quote_description_create(void)
 	vbox = gtk_vbox_new(FALSE, 8);
 	gtk_container_add(GTK_CONTAINER(quote_desc_win), vbox);
 
-	hbox = gtk_hbox_new(FALSE, 4);
+	hbox = gtk_hbox_new(FALSE, 8);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
 
 	label = gtk_label_new
-		("%d:\n"
-		 "%f:\n"
-		 "%n:\n"
-		 "%N:\n"
-		 "%I:\n"
-		 "%s:\n"
-		 "%t:\n"
-		 "%c:\n"
-		 "%i:\n"
-		 "%%:");
+		("%d\n"		/* date */
+		 "%f\n"		/* from */
+		 "%N\n"		/* full name of sender */
+		 "%F\n"		/* first name of sender */
+		 "%I\n"		/* initial of sender */
+		 "%s\n"		/* subject */
+		 "%t\n"		/* to */
+		 "%c\n"		/* cc */
+		 "%n\n"		/* newsgroups */
+		 "%i\n"		/* message id */
+		 "\n"
+		 "?x(expr)\n"	/* condition */
+		 "\n"
+		 "%M\n"		/* message body */
+		 "%Q\n"		/* quoted message body */
+		 "%m\n"		/* message body without signature */
+		 "%q\n"		/* quoted message body without signature */
+		 "%%");		/* literal percent */
 
 	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
 	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
@@ -2428,8 +2600,16 @@ static void prefs_quote_description_create(void)
 		   "Subject\n"
 		   "To\n"
 		   "Cc\n"
+		   "Newsgroups\n"
 		   "Message-ID\n"
-		   "%"));
+		   "\n"
+		   "If x is set, displays expr\n"
+		   "\n"
+		   "Message body\n"
+		   "Quoted message body\n"
+		   "Message body without signature\n"
+		   "Quoted message body without signature\n"
+		   "Literal %"));
 
 	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
 	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
@@ -2440,161 +2620,23 @@ static void prefs_quote_description_create(void)
 
 	gtk_widget_grab_default(ok_btn);
 	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
-				  GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
-
+			   GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+	gtk_signal_connect
+		(GTK_OBJECT(quote_desc_win), "key_press_event",
+		 GTK_SIGNAL_FUNC(prefs_quote_description_key_pressed),
+		 NULL);
 	gtk_signal_connect(GTK_OBJECT(quote_desc_win), "delete_event",
-					  GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+			   GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
 
 	gtk_widget_show_all(vbox);
 }
 
-/* functions for setting items of SummaryView */
-
-static struct _SummaryDisplayItem
+static void prefs_quote_description_key_pressed(GtkWidget *widget,
+						GdkEventKey *event,
+						gpointer data)
 {
-	GtkWidget *window;
-
-	GtkWidget *chkbtn[N_SUMMARY_COLS];
-
-	GtkWidget *ok_btn;
-	GtkWidget *cancel_btn;
-} summarydispitem;
-
-#define SET_ACTIVE(column, var) \
-	gtk_toggle_button_set_active \
-		(GTK_TOGGLE_BUTTON(summarydispitem.chkbtn[column]), \
-		 prefs_common.var)
-#define GET_ACTIVE(column, var) \
-	prefs_common.var = gtk_toggle_button_get_active \
-		(GTK_TOGGLE_BUTTON(summarydispitem.chkbtn[column]))
-
-void prefs_summary_display_item_set(void)
-{
-	static gboolean cancelled;
-
-	if (!summarydispitem.window)
-		prefs_summary_display_item_dialog_create(&cancelled);
-	gtk_widget_grab_focus(summarydispitem.ok_btn);
-	gtk_widget_show(summarydispitem.window);
-	manage_window_set_transient(GTK_WINDOW(summarydispitem.window));
-
-	SET_ACTIVE(S_COL_MARK, show_mark);
-	SET_ACTIVE(S_COL_UNREAD, show_unread);
-	SET_ACTIVE(S_COL_MIME, show_mime);
-	SET_ACTIVE(S_COL_NUMBER, show_number);
-	SET_ACTIVE(S_COL_SIZE, show_size);
-	SET_ACTIVE(S_COL_DATE, show_date);
-	SET_ACTIVE(S_COL_FROM, show_from);
-	SET_ACTIVE(S_COL_SUBJECT, show_subject);
-
-	gtk_main();
-	gtk_widget_hide(summarydispitem.window);
-
-	if (cancelled != TRUE) {
-		GET_ACTIVE(S_COL_MARK, show_mark);
-		GET_ACTIVE(S_COL_UNREAD, show_unread);
-		GET_ACTIVE(S_COL_MIME, show_mime);
-		GET_ACTIVE(S_COL_NUMBER, show_number);
-		GET_ACTIVE(S_COL_SIZE, show_size);
-		GET_ACTIVE(S_COL_DATE, show_date);
-		GET_ACTIVE(S_COL_FROM, show_from);
-		GET_ACTIVE(S_COL_SUBJECT, show_subject);
-
-		main_window_reflect_prefs_all();
-	}
-}
-
-#define SET_CHECK_BUTTON(column, label) \
-{ \
-	summarydispitem.chkbtn[column] = \
-		gtk_check_button_new_with_label(label); \
-	gtk_box_pack_start(GTK_BOX(chkbtn_vbox), \
-			   summarydispitem.chkbtn[column], \
-			   FALSE, FALSE, 0); \
-}
-
-static void prefs_summary_display_item_dialog_create(gboolean *cancelled)
-{
-	GtkWidget *window;
-	GtkWidget *vbox;
-	GtkWidget *chkbtn_vbox;
-	GtkWidget *hbbox;
-	GtkWidget *ok_btn;
-	GtkWidget *cancel_btn;
-
-	window = gtk_window_new(GTK_WINDOW_DIALOG);
-	gtk_container_set_border_width(GTK_CONTAINER(window), 8);
-	gtk_window_set_title(GTK_WINDOW(window), _("Set display item"));
-	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
-	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
-	gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(display_item_delete_event),
-			   cancelled);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(display_item_key_pressed),
-			   cancelled);
-
-	vbox = gtk_vbox_new(FALSE, 8);
-	gtk_container_add(GTK_CONTAINER(window), vbox);
-
-	chkbtn_vbox = gtk_vbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), chkbtn_vbox, FALSE, FALSE, 0);
-
-	SET_CHECK_BUTTON(S_COL_MARK, _("Mark"));
-	SET_CHECK_BUTTON(S_COL_UNREAD, _("Unread"));
-	SET_CHECK_BUTTON(S_COL_MIME, _("MIME"));
-	SET_CHECK_BUTTON(S_COL_NUMBER, _("Number"));
-	SET_CHECK_BUTTON(S_COL_SIZE, _("Size"));
-	SET_CHECK_BUTTON(S_COL_DATE, _("Date"));
-	SET_CHECK_BUTTON(S_COL_FROM, _("From"));
-	SET_CHECK_BUTTON(S_COL_SUBJECT, _("Subject"));
-
-	gtkut_button_set_create(&hbbox, &ok_btn, _("OK"),
-				&cancel_btn, _("Cancel"), NULL, NULL);
-	gtk_box_pack_end(GTK_BOX(vbox), hbbox, FALSE, FALSE, 0);
-	gtk_widget_grab_default(ok_btn);
-
-	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
-			   GTK_SIGNAL_FUNC(display_item_ok), cancelled);
-	gtk_signal_connect(GTK_OBJECT(cancel_btn), "clicked",
-			   GTK_SIGNAL_FUNC(display_item_cancel), cancelled);
-
-	gtk_widget_show_all(vbox);
-
-	summarydispitem.window = window;
-	summarydispitem.ok_btn = ok_btn;
-	summarydispitem.cancel_btn = cancel_btn;
-}
-
-static void display_item_ok(GtkWidget *widget, gboolean *cancelled)
-{
-	*cancelled = FALSE;
-	gtk_main_quit();
-}
-
-static void display_item_cancel(GtkWidget *widget, gboolean *cancelled)
-{
-	*cancelled = TRUE;
-	gtk_main_quit();
-}
-
-static gint display_item_delete_event(GtkWidget *widget, GdkEventAny *event,
-				      gboolean *cancelled)
-{
-	*cancelled = TRUE;
-	gtk_main_quit();
-
-	return TRUE;
-}
-
-static void display_item_key_pressed(GtkWidget *widget, GdkEventKey *event,
-				     gboolean *cancelled)
-{
-	if (event && event->keyval == GDK_Escape) {
-		*cancelled = TRUE;
+	if (event && event->keyval == GDK_Escape)
 		gtk_main_quit();
-	}
 }
 
 static void prefs_font_select(GtkButton *button)
