@@ -27,6 +27,7 @@
 #include <glib.h>
 
 #include "mainwindow.h"
+#include "summaryview.h"
 #include "codeconv.h"
 #include "textview.h"
 
@@ -60,9 +61,6 @@ struct _PrefsCommon
 	gchar *outgoing_charset;
 
 	/* Compose */
-	gboolean reply_with_quote;
-	gchar *quotemark;
-	gchar *quotefmt;
 	gboolean auto_sig;
 	gchar *sig_sep;
 	gint linewrap_len;
@@ -72,6 +70,13 @@ struct _PrefsCommon
 	gboolean reply_account_autosel;
 	gboolean show_ruler;
 
+	/* Quote */
+	gboolean reply_with_quote;
+	gchar *quotemark;
+	gchar *quotefmt;
+	gchar *fw_quotemark;
+	gchar *fw_quotefmt;
+
 	/* Display */
 	gchar *widgetfont;
 	gchar *textfont;
@@ -92,14 +97,10 @@ struct _PrefsCommon
 	/* Filtering */
 	GSList *fltlist;
 
-	gboolean show_mark;
-	gboolean show_unread;
-	gboolean show_mime;
-	gboolean show_number;
-	gboolean show_size;
-	gboolean show_date;
-	gboolean show_from;
-	gboolean show_subject;
+	/* Summary columns visibility, position and size */
+	gboolean summary_col_visible[N_SUMMARY_COLS];
+	gint summary_col_pos[N_SUMMARY_COLS];
+	gint summary_col_size[N_SUMMARY_COLS];
 
 	/* Widget size */
 	gint folderview_x;
@@ -113,14 +114,6 @@ struct _PrefsCommon
 
 	gint summaryview_width;
 	gint summaryview_height;
-	gint summary_col_mark;
-	gint summary_col_unread;
-	gint summary_col_mime;
-	gint summary_col_number;
-	gint summary_col_size;
-	gint summary_col_date;
-	gint summary_col_from;
-	gint summary_col_subject;
 
 	gint mainview_x;
 	gint mainview_y;
@@ -162,6 +155,9 @@ struct _PrefsCommon
 	/* MIME viewer */
 	gchar *mime_image_viewer;
 	gchar *mime_audio_player;
+	gchar *mime_open_cmd;
+
+	GList *mime_open_cmd_history;
 
 	/* Privacy */
 	gboolean default_encrypt;
@@ -199,6 +195,6 @@ void prefs_common_read_config	(void);
 void prefs_common_save_config	(void);
 void prefs_common_open		(void);
 
-void prefs_summary_display_item_set	(void);
+void prefs_quote_description	(void);
 
 #endif /* __PREFS_COMMON_H__ */
