@@ -99,6 +99,9 @@ static void prefs_filter_select		(GtkCList	*clist,
 					 gint		 row,
 					 gint		 column,
 					 GdkEvent	*event);
+static void prefs_filter_row_move	(GtkCList	*clist,
+					 gint		 source_row,
+					 gint		 dest_row);
 
 static void prefs_filter_dest_radio_button_toggled	(void);
 static void prefs_filter_notrecv_radio_button_toggled	(void);
@@ -419,6 +422,8 @@ static void prefs_filter_create(void)
 				GTK_CAN_FOCUS);
 	gtk_signal_connect (GTK_OBJECT (cond_clist), "select_row",
 			    GTK_SIGNAL_FUNC (prefs_filter_select), NULL);
+	gtk_signal_connect (GTK_OBJECT (cond_clist), "row_move",
+			    GTK_SIGNAL_FUNC (prefs_filter_row_move), NULL);
 
 	btn_vbox = gtk_vbox_new (FALSE, 8);
 	gtk_widget_show (btn_vbox);
@@ -730,10 +735,8 @@ static void prefs_filter_up(void)
 	if (!clist->selection) return;
 
 	row = GPOINTER_TO_INT(clist->selection->data);
-	if (row > 1) {
+	if (row > 1)
 		gtk_clist_row_move(clist, row, row - 1);
-		prefs_filter_set_list();
-	}
 }
 
 static void prefs_filter_down(void)
@@ -744,10 +747,8 @@ static void prefs_filter_down(void)
 	if (!clist->selection) return;
 
 	row = GPOINTER_TO_INT(clist->selection->data);
-	if (row > 0 && row < clist->rows - 1) {
+	if (row > 0 && row < clist->rows - 1)
 		gtk_clist_row_move(clist, row, row + 1);
-		prefs_filter_set_list();
-	}
 }
 
 #define ENTRY_SET_TEXT(entry, str) \
@@ -789,6 +790,16 @@ static void prefs_filter_select(GtkCList *clist, gint row, gint column,
 			(GTK_TOGGLE_BUTTON(filter.dest_radiobtn), TRUE);
 }
 
+static void prefs_filter_row_move(GtkCList *clist, gint source_row,
+				  gint dest_row)
+{
+	prefs_filter_set_list();
+	if (gtk_clist_row_is_visible(clist, dest_row) != GTK_VISIBILITY_FULL) {
+		gtk_clist_moveto(clist, dest_row, -1,
+				 source_row < dest_row ? 1.0 : 0.0, 0.0);
+	}
+}
+
 static void prefs_filter_dest_radio_button_toggled(void)
 {
 	gtk_widget_set_sensitive(filter.dest_entry, TRUE);
