@@ -467,6 +467,7 @@ MsgInfo *procheader_parse(const gchar *file, MsgFlags flags, gboolean full)
 			msginfo->msgid = g_strdup(hp);
 			break;
 		case H_REFERENCES:
+		case H_IN_REPLY_TO:
 			if (!reference) {
 				eliminate_parenthesis(hp, '(', ')');
 				if ((p = strrchr(hp, '<')) != NULL &&
@@ -478,15 +479,6 @@ MsgInfo *procheader_parse(const gchar *file, MsgFlags flags, gboolean full)
 				}
 			}
 			break;
-		case H_IN_REPLY_TO:
-			if (!reference) {
-				eliminate_parenthesis(hp, '(', ')');
-				extract_parenthesis(hp, '<', '>');
-				remove_space(hp);
-				if (*hp != '\0')
-					reference = g_strdup(hp);
-			}
-			break;
 		case H_CONTENT_TYPE:
 			if (!strncasecmp(hp, "multipart", 9))
 				MSG_SET_TMP_FLAGS(msginfo->flags, MSG_MIME);
@@ -513,8 +505,7 @@ gchar *procheader_get_fromname(const gchar *str)
 {
 	gchar *tmp, *name;
 
-	Xalloca(tmp, strlen(str) + 1, return NULL);
-	strcpy(tmp, str);
+	Xstrdup_a(tmp, str, return NULL);
 
 	if (*tmp == '\"') {
 		extract_quote(tmp, '\"');
