@@ -182,15 +182,15 @@ TextView *textview_create(void)
 	gtk_signal_connect(GTK_OBJECT(text_sb), "key_press_event",
 			   GTK_SIGNAL_FUNC(textview_key_pressed),
 			   textview);
-	gtk_signal_connect(GTK_OBJECT(text_sb), "button_press_event",
-			   GTK_SIGNAL_FUNC(textview_button_pressed),
-			   textview);
+	gtk_signal_connect_after(GTK_OBJECT(text_sb), "button_press_event",
+				 GTK_SIGNAL_FUNC(textview_button_pressed),
+				 textview);
 	gtk_signal_connect(GTK_OBJECT(text_mb), "key_press_event",
 			   GTK_SIGNAL_FUNC(textview_key_pressed),
 			   textview);
-	gtk_signal_connect(GTK_OBJECT(text_mb), "button_press_event",
-			   GTK_SIGNAL_FUNC(textview_button_pressed),
-			   textview);
+	gtk_signal_connect_after(GTK_OBJECT(text_mb), "button_press_event",
+				 GTK_SIGNAL_FUNC(textview_button_pressed),
+				 textview);
 
 	gtk_widget_show(scrolledwin_sb);
 	gtk_widget_show(scrolledwin_mb);
@@ -207,6 +207,8 @@ TextView *textview_create(void)
 	textview->text_mb        = text_mb;
 	textview->text_is_mb     = FALSE;
 	textview->uri_list       = NULL;
+	textview->body_pos       = 0;
+	textview->cur_pos        = 0;
 
 	return textview;
 }
@@ -332,6 +334,9 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	text = GTK_TEXT(textview->text);
 	gtk_text_freeze(text);
 
+	textview->body_pos = 0;
+	textview->cur_pos  = 0;
+
 	if (headers) {
 		textview_show_header(textview, headers);
 		procheader_header_array_destroy(headers);
@@ -662,7 +667,6 @@ static void textview_write_line(TextView *textview, const gchar *str,
 {
 	GtkText *text = GTK_TEXT(textview->text);
 	gchar buf[BUFFSIZE];
-	size_t len;
 	GdkColor *fg_color;
 	gint quotelevel = -1;
 
@@ -677,11 +681,7 @@ static void textview_write_line(TextView *textview, const gchar *str,
 		return;
 	}
 
-	len = strlen(buf);
-	if (len > 1 && buf[len - 1] == '\n' && buf[len - 2] == '\r') {
-		buf[len - 2] = '\n';
-		buf[len - 1] = '\0';
-	}
+	strcrchomp(buf);
 	if (prefs_common.conv_mb_alnum) conv_mb_alnum(buf);
 	fg_color = NULL;
 
@@ -723,7 +723,8 @@ void textview_clear(TextView *textview)
 	GtkText *text = GTK_TEXT(textview->text);
 
 	gtk_text_freeze(text);
-	gtk_text_backward_delete(text, gtk_text_get_length(text));
+	gtk_text_set_point(text, 0);
+	gtk_text_forward_delete(text, gtk_text_get_length(text));
 	gtk_text_thaw(text);
 
 	textview_uri_list_remove_all(textview->uri_list);
@@ -760,11 +761,13 @@ void textview_set_font(TextView *textview, const gchar *codeset)
 	if (MB_CUR_MAX > 1) {
 		if (codeset) {
 			if (!g_strncasecmp(codeset, "ISO-8859-", 9) ||
-			    !g_strncasecmp(codeset, "KOI8-", 5)     ||
-			    !g_strncasecmp(codeset, "CP", 2)        ||
-			    !g_strncasecmp(codeset, "WINDOWS-", 8)  ||
 			    !g_strcasecmp(codeset, "BALTIC"))
 				use_fontset = FALSE;
+			else if (conv_get_current_charset() != C_EUC_JP &&
+				 (!g_strncasecmp(codeset, "KOI8-", 5) ||
+				  !g_strncasecmp(codeset, "CP", 2)    ||
+				  !g_strncasecmp(codeset, "WINDOWS-", 8)))
+				use_fontset = FALSE;
 		}
 	} else
 		use_fontset = FALSE;
@@ -835,6 +838,16 @@ void textview_set_font(TextView *textview, const gchar *codeset)
 		spacingfont = gdk_font_load("-*-*-medium-r-normal--6-*");
 }
 
+void textview_set_position(TextView *textview, gint pos)
+{
+	if (pos < 0) {
+		textview->cur_pos =
+			gtk_text_get_length(GTK_TEXT(textview->text));
+	} else {
+		textview->cur_pos = pos;
+	}
+}
+
 static GPtrArray *textview_scan_header(TextView *textview, FILE *fp)
 {
 	gchar buf[BUFFSIZE];
@@ -931,6 +944,91 @@ static void textview_show_header(TextView *textview, GPtrArray *headers)
 
 	gtk_text_insert(text, textview->msgfont, NULL, NULL, "\n", 1);
 	gtk_text_thaw(text);
+	textview->body_pos = gtk_text_get_length(text);
+}
+
+gboolean textview_search_string(TextView *textview, const gchar *str,
+				gboolean case_sens)
+{
+	GtkText *text = GTK_TEXT(textview->text);
+	gint pos;
+	wchar_t *wcs;
+	gint len;
+	gint text_len;
+	gboolean found = FALSE;
+
+	g_return_val_if_fail(str != NULL, FALSE);
+
+	wcs = strdup_mbstowcs(str);
+	g_return_val_if_fail(wcs != NULL, FALSE);
+	len = wcslen(wcs);
+	pos = textview->cur_pos;
+	if (pos < textview->body_pos)
+		pos = textview->body_pos;
+	text_len = gtk_text_get_length(text);
+	if (text_len - pos < len) {
+		g_free(wcs);
+		return FALSE;
+	}
+
+	for (; pos < text_len; pos++) {
+		if (text_len - pos < len) break;
+		if (gtkut_text_match_string(text, pos, wcs, len, case_sens)
+		    == TRUE) {
+			gtk_editable_set_position(GTK_EDITABLE(text),
+						  pos + len);
+			gtk_editable_select_region(GTK_EDITABLE(text),
+						   pos, pos + len);
+			textview_set_position(textview, pos + len);
+			found = TRUE;
+			break;
+		}
+		if (text_len - pos == len) break;
+	}
+
+	g_free(wcs);
+	return found;
+}
+
+gboolean textview_search_string_backward(TextView *textview, const gchar *str,
+					 gboolean case_sens)
+{
+	GtkText *text = GTK_TEXT(textview->text);
+	gint pos;
+	wchar_t *wcs;
+	gint len;
+	gint text_len;
+	gboolean found = FALSE;
+
+	g_return_val_if_fail(str != NULL, FALSE);
+
+	wcs = strdup_mbstowcs(str);
+	g_return_val_if_fail(wcs != NULL, FALSE);
+	len = wcslen(wcs);
+	pos = textview->cur_pos;
+	text_len = gtk_text_get_length(text);
+	if (text_len - textview->body_pos < len) {
+		g_free(wcs);
+		return FALSE;
+	}
+	if (pos <= textview->body_pos || text_len - pos < len)
+		pos = text_len - len;
+
+	for (; pos >= textview->body_pos; pos--) {
+		if (gtkut_text_match_string(text, pos, wcs, len, case_sens)
+		    == TRUE) {
+			gtk_editable_set_position(GTK_EDITABLE(text), pos);
+			gtk_editable_select_region(GTK_EDITABLE(text),
+						   pos, pos + len);
+			textview_set_position(textview, pos - 1);
+			found = TRUE;
+			break;
+		}
+		if (pos == textview->body_pos) break;
+	}
+
+	g_free(wcs);
+	return found;
 }
 
 void textview_scroll_one_line(TextView *textview, gboolean up)
@@ -1135,7 +1233,6 @@ static void textview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 			textview_scroll_page(textview, FALSE);
 		break;
 	case GDK_BackSpace:
-	case GDK_Delete:
 		textview_scroll_page(textview, TRUE);
 		break;
 	case GDK_Return:
@@ -1152,19 +1249,19 @@ static void textview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 static void textview_button_pressed(GtkWidget *widget, GdkEventButton *event,
 				    TextView *textview)
 {
+	textview->cur_pos = 
+		gtk_editable_get_position(GTK_EDITABLE(textview->text));
+
 	if (event &&
 	    ((event->button == 1 && event->type == GDK_2BUTTON_PRESS) ||
 	     event->button == 2)) {
 		GSList *cur;
-		guint current_pos;
-
-		current_pos = GTK_EDITABLE(textview->text)->current_pos;
 
 		for (cur = textview->uri_list; cur != NULL; cur = cur->next) {
 			RemoteURI *uri = (RemoteURI *)cur->data;
 
-			if (current_pos >= uri->start &&
-			    current_pos <  uri->end) {
+			if (textview->cur_pos >= uri->start &&
+			    textview->cur_pos <  uri->end) {
 				if (!g_strncasecmp(uri->uri, "mailto:", 7))
 					compose_new_with_recipient
 						(NULL, uri->uri + 7);
