@@ -136,26 +136,15 @@ gchar *itos(gint n)
 gchar *to_human_readable(off_t size)
 {
 	static gchar str[10];
-	gint count;
-	guint32 div = 1;
 
-	for (count = 0; count < 3; count++) {
-		if (size / div < 1024)
-			break;
-		else
-			div *= 1024;
-	}
-
-	switch (count) {
-	case 0: g_snprintf(str, sizeof(str), "%dB",    (gint)size);   break;
-	case 1: g_snprintf(str, sizeof(str), "%.1fKB", (gfloat)size / div);
-		break;
-	case 2: g_snprintf(str, sizeof(str), "%.2fMB", (gfloat)size / div);
-		break;
-	default:
-		g_snprintf(str, sizeof(str), "%.2fGB", (gfloat)size / div);
-		break;
-	}
+	if (size < 1024)
+		g_snprintf(str, sizeof(str), "%dB", (gint)size);
+	else if (size >> 10 < 1024)
+		g_snprintf(str, sizeof(str), "%.1fKB", (gfloat)size / (1 << 10));
+	else if (size >> 20 < 1024)
+		g_snprintf(str, sizeof(str), "%.2fMB", (gfloat)size / (1 << 20));
+	else
+		g_snprintf(str, sizeof(str), "%.2fGB", (gfloat)size / (1 << 30));
 
 	return str;
 }
@@ -215,6 +204,21 @@ gchar *strtailchomp(gchar *str, gchar tail_char)
 	return str;
 }
 
+/* remove CR (carriage return) */
+gchar *strcrchomp(gchar *str)
+{
+	register gchar *s;
+
+	if (!*str) return str;
+
+	s = str + strlen(str) - 1;
+	if (*s == '\n' && s > str && *(s - 1) == '\r') {
+		*(s - 1) = '\n';
+		*s = '\0';
+	}
+
+	return str;
+}
 
 /* Similar to `strstr' but this function ignores the case of both strings.  */
 gchar *strcasestr(const gchar *haystack, const gchar *needle)
@@ -462,7 +466,7 @@ gboolean is_next_mbs(const wchar_t *s)
 {
 	gint mbl;
 	const wchar_t *wp;
-	gchar tmp[MB_CUR_MAX];
+	gchar tmp[MB_LEN_MAX];
 
 	/* skip head space */
 	for (wp = s; *wp != (wchar_t)0 && iswspace(*wp); wp++)
@@ -493,29 +497,20 @@ wchar_t *find_wspace(const wchar_t *s)
 /* compare subjects */
 gint subject_compare(const gchar *s1, const gchar *s2)
 {
-	gint retval;
 	gchar *str1, *str2;
 
 	if (!s1 || !s2) return -1;
 	if (!*s1 || !*s2) return -1;
 
-	Xalloca(str1, strlen(s1) + 1, return -1);
-	Xalloca(str2, strlen(s2) + 1, return -1);
-	strcpy(str1, s1);
-	strcpy(str2, s2);
+	Xstrdup_a(str1, s1, return -1);
+	Xstrdup_a(str2, s2, return -1);
 
 	trim_subject(str1);
 	trim_subject(str2);
 
 	if (!*str1 || !*str2) return -1;
 
-	retval = strcmp(str1, str2);
-	//if (retval == 0)
-	//	g_print("\ns1 = %s\ns2 = %s\n"
-	//		"str1 = %s\nstr2 = %s\nmatched.\n",
-	//		s1, s2, str1, str2);
-
-	return retval;
+	return strcmp(str1, str2);
 }
 
 void trim_subject(gchar *str)
@@ -851,6 +846,31 @@ GSList *newsgroup_list_append(GSList *group_list, const gchar *str)
 	return group_list;
 }
 
+GList *add_history(GList *list, const gchar *str)
+{
+	GList *old;
+
+	g_return_val_if_fail(str != NULL, list);
+
+	old = g_list_find_custom(list, (gpointer)str, (GCompareFunc)strcmp2);
+	if (old) {
+		g_free(old->data);
+		list = g_list_remove(list, old->data);
+	} else if (g_list_length(list) >= MAX_HISTORY_SIZE) {
+		GList *last;
+
+		last = g_list_last(list);
+		if (last) {
+			g_free(last->data);
+			g_list_remove(list, last->data);
+		}
+	}
+
+	list = g_list_prepend(list, g_strdup(str));
+
+	return list;
+}
+
 void remove_return(gchar *str)
 {
 	register gchar *p = str;
@@ -1173,6 +1193,17 @@ gchar *get_mime_tmp_dir(void)
 	return mime_tmp_dir;
 }
 
+gchar *get_template_dir(void)
+{
+	static gchar *template_dir = NULL;
+
+	if (!template_dir)
+		template_dir = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
+					   TEMPLATE_DIR, NULL);
+
+	return template_dir;
+}
+
 gchar *get_tmp_file(void)
 {
 	static gchar *tmp_file = NULL;
@@ -1419,7 +1450,7 @@ gint remove_dir_recursive(const gchar *dir)
 	struct dirent *d;
 	gchar *prev_dir;
 
-	//g_print("dir = %s\n", dir);
+	/* g_print("dir = %s\n", dir); */
 
 	if (stat(dir, &s) < 0) {
 		FILE_OP_ERROR(dir, "stat");
@@ -1437,7 +1468,7 @@ gint remove_dir_recursive(const gchar *dir)
 	}
 
 	prev_dir = g_get_current_dir();
-	//g_print("prev_dir = %s\n", prev_dir);
+	/* g_print("prev_dir = %s\n", prev_dir); */
 
 	if (!path_cmp(prev_dir, dir)) {
 		g_free(prev_dir);
@@ -1472,7 +1503,7 @@ gint remove_dir_recursive(const gchar *dir)
 			continue;
 		}
 
-		//g_print("removing %s\n", d->d_name);
+		/* g_print("removing %s\n", d->d_name); */
 
 		if (S_ISDIR(s.st_mode)) {
 			if (remove_dir_recursive(d->d_name) < 0) {
